package adressa;

import global.GlobalParameters;

import util.LogHelper;
import weka.core.Instances;
import weka.core.converters.ConverterUtils.DataSource;
import weka.core.Utils;
import weka.classifiers.Classifier;
import weka.classifiers.Evaluation;
import weka.classifiers.bayes.NaiveBayes;
import weka.classifiers.bayes.NaiveBayesMultinomial;
import weka.classifiers.functions.SMO;
import weka.core.converters.TextDirectoryLoader;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.StringToWordVector;

import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Random;



public class AdressaClassifier {

	public AdressaClassifier(){

	}

	public double runClassifier() throws Exception{

		TextDirectoryLoader loader = new TextDirectoryLoader();
		loader.setDirectory(new File("resources/adressa/wRAKE"));
		Instances dataRaw = loader.getDataSet();  // contains the full dataset we wann create train/test sets from
		int folds = 5;         // the number of folds to generate, >=2
		int runs = 10;

		StringToWordVector filter = new StringToWordVector();

		String[] filterOptArr = new String[2];
		filterOptArr[0] = "-N";
		filterOptArr[1] = "1";
		filter.setInputFormat(dataRaw);
//		filter.setOutputWordCounts(true);
//		filter.setOptions(filterOptArr);
		String[] optArr = new String[2];
		optArr[0] = "-delimiters";
		optArr[1] = ",";
		filter.getTokenizer().setOptions(optArr);
		for(String s: filter.getOptions()){
			System.out.println(s);
		}
		Instances dataFiltered = Filter.useFilter(dataRaw, filter);
		Classifier cls = new NaiveBayesMultinomial();


		double avgCorrect=0;
//		Evaluation evalTop = new Evaluation(dataFiltered);
		for (int i = 0; i < runs; i++) {
			int seed = i+1;
			Random rand = new Random(seed);   // create seeded number generator
			Instances randData = new Instances(dataFiltered);   // create copy of original data
			randData.randomize(rand);         // randomize data with number generator

			if (randData.classAttribute().isNominal())
				randData.stratify(folds);

			Evaluation eval = new Evaluation(randData);

			for (int n = 0; n < folds; n++) {
				Instances train = randData.trainCV(folds, n);
				Instances test = randData.testCV(folds, n);
				//				randData.delete();

				Classifier clsCopy = Classifier.makeCopy(cls);
				clsCopy.buildClassifier(train);
				eval.evaluateModel(clsCopy, test);
//				evalTop.evaluateModel(clsCopy, test);
			}
			avgCorrect+=eval.pctCorrect();
			// output evaluation
		      System.out.println();
		      System.out.println("=== Setup run " + (i+1) + " ===");
		      System.out.println("Classifier: " + cls.getClass().getName() + " " + Utils.joinOptions(cls.getOptions()));
		      System.out.println("Dataset: " + dataRaw.relationName());
		      System.out.println("Folds: " + folds);
		      System.out.println("Seed: " + seed);
		      System.out.println();
		      System.out.println(eval.toSummaryString("=== " + folds + "-fold Cross-validation run " + (i+1) + "===", false));

		}
		avgCorrect=avgCorrect/runs;
		System.out.println("AverageCorrect: " + avgCorrect);
		System.out.println("StopwordList: " + GlobalParameters.STOP_WORD_LIST);
		return avgCorrect;
		// output evaluation
//	      System.out.println();
//	      System.out.println("=== Setup run total ===");
//	      System.out.println("Classifier: " + cls.getClass().getName() + " " + Utils.joinOptions(cls.getOptions()));
//	      System.out.println("Dataset: " + dataRaw.relationName());
//	      System.out.println("Folds: " + folds);
//	      System.out.println();
//	      System.out.println(evalTop.toSummaryString("=== " + folds + "-fold Cross-validation run total ===", false));


	}


	public static void main(String[] args) throws Exception {
		AdressaClassifier aClass = new AdressaClassifier();
		File dir = new File(GlobalParameters.LOGFILE+aClass.getClass().getName());
		if(!dir.isDirectory()){
			dir.mkdir();
		}
		PrintStream ps = new PrintStream(new File(LogHelper.getLogName()));
		System.setOut(ps);
		GlobalParameters.printOptions();
		aClass.runClassifier();
	}

}
