package adressa;

import taggerAndWordbank.impl.OsloBergenTagger;

public class AdressaDocument {
	
	private String urlString;
	private String titleString;
	private String headerString;
	private String bodyString;
	private String tagString;
	
	private String documentString;
	
	private OsloBergenTagger obt;
	
	public AdressaDocument(String urlStringString, String titleString, String headerString, String bodyString, String tagString){
		this.urlString = urlStringString;
		this.titleString = titleString;
		this.tagString = tagString;
		this.bodyString = bodyString;
		this.headerString = headerString;
		
		generateDocumentString();
	}
	
	public String getDocumentString() {
		return documentString;
	}

	public void setDocumentString(String documentString) {
		this.documentString = documentString;
	}

	private void generateDocumentString(){
		String key = generateStarBrackets("TAGS", tagString);
		String title = generateStarBrackets("TITLE", titleString);
		String header = generateStarBrackets("HEADER", headerString);
		String body = bodyString.replace(headerString, "");
		documentString = key + "\n" + title + "\n" + header + "\n" + body;
	}
	
	public String generateTaggedDocumentString(OsloBergenTagger obt){
		this.obt = obt;
		String key = generateStarBrackets("TAGS", getTaggedString(tagString));
		String title = generateStarBrackets("TITLE", getTaggedString(titleString));
		String header = generateStarBrackets("HEADER", getTaggedString(headerString));
		String body = getTaggedString(bodyString.replace(headerString, ""));
		return key + "\n" + title + "\n" + header + "\n" + body;
	}
	
	public String getTaggedString(String text){
		if(text==null || text.isEmpty()){
			return null;
		}
		String[] tags = null;
		String[] terms = null;
		try{
			tags = obt.tag(new StringBuilder(text));
			terms = obt.getTerms();
		}catch(NullPointerException e){
			System.out.println(text);
			e.printStackTrace();
		}
			
		String returnString="";
		
		if (tags != null && terms != null && tags.length == terms.length){
			for (int i = 0; i < tags.length; i++){
				returnString += obt.fixUnknownChars(terms[i]) + "/" + tags[i] + (terms[i].equals(".") ? "\n" : " ");
			}
		}
		return returnString;
	}
	
	private String generateStarBrackets(String type, String value){
		return "*"+type+"*" + value + "*/" + type + "*";
	}

	public String getUrlString() {
		return urlString;
	}

	public void setUrlString(String urlString) {
		this.urlString = urlString;
	}

	public String getTitleString() {
		return titleString;
	}

	public void setTitleString(String titleString) {
		this.titleString = titleString;
	}

	public String getHeaderString() {
		return headerString;
	}

	public void setHeaderString(String headerString) {
		this.headerString = headerString;
	}

	public String getBodyString() {
		return bodyString;
	}

	public void setBodyString(String bodyString) {
		this.bodyString = bodyString;
	}

	public String getTagString() {
		return tagString;
	}

	public void setTagString(String tagString) {
		this.tagString = tagString;
	}
	
	

}
