package adressa;

import java.util.ArrayList;
import java.util.List;

import javax.swing.text.html.HTML.Attribute;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class AdressaSAXParser {
	
	private String urlString;
	private String titleString;
	private String headerString;
	private String bodyString;
	private String tagString;
	
	private List<AdressaDocument> docs = new ArrayList<AdressaDocument>();
	private String nextPageUrl;
	

	public void parse(String xmlUrl){
		try{
			SAXParserFactory factory = SAXParserFactory.newInstance();
			SAXParser saxParser = factory.newSAXParser();

			DefaultHandler handler = new DefaultHandler() {
				
				boolean url = false;
				boolean title = false;
				boolean header = false;
				boolean body = false;
				boolean tag = false;

				public void startElement(String uri, String localName,
						String qName, Attributes attributes)
				throws SAXException {

//					System.out.println("Start Element :" + qName);

					if(qName.equalsIgnoreCase("FIELD")){
						if(attributes.getValue(0).equalsIgnoreCase("contentid")){
							url = true;
						}
						if(attributes.getValue(0).equalsIgnoreCase("title")){
							title = true;
						}
						if(attributes.getValue(0).equalsIgnoreCase("teaser")){
							header = true;
						}
						if(attributes.getValue(0).equalsIgnoreCase("iadcontent")){
							body = true;
						}
						if(attributes.getValue(0).equalsIgnoreCase("iadtags")){
							tag = true;
						}
						
					}
					else if(qName.equalsIgnoreCase("NEXTPAGE")){
						nextPageUrl = attributes.getValue(2);
					}

				}

				public void endElement(String uri, String localName,
						String qName)
				throws SAXException {
					if(qName.equalsIgnoreCase("FIELD")){
						if(bodyString!=null && !bodyString.isEmpty()){
							docs.add(new AdressaDocument(urlString, titleString, headerString, bodyString, tagString));
							urlString="";
							titleString="";
							headerString="";
							bodyString="";
							tagString="";
						}
					}


				}

				public void characters(char ch[], int start, int length)
				throws SAXException {
					
					if(url){
						urlString = new String(ch, start, length);
//						System.out.println("Url : "
//								+ new String(ch, start, length));
						url = false;
					}
					if(title){
						titleString = new String(ch, start, length);
						title = false;
					}
					if(header){
						headerString = new String(ch, start, length);
						header = false;
					}
					if(body){
						bodyString = new String(ch, start, length);
						body = false;
					}
					if(tag){
						tagString = new String(ch, start, length);
						tag = false;
					}

				}

			};

//			saxParser.parse("C:\\Users\\Kristian\\Desktop\\xsearch.xml", handler);
//			saxParser.parse("http://slt.idi.ntnu.no:15100/cgi-bin/xsearch?query=docdatetime%3arange%28min%2c+2011-02-02%29&hits=10&view=adressa&rpf%5fnavigation%3aenabled=0&offset=10", handler);
			saxParser.parse(xmlUrl, handler);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	

	
	public List<AdressaDocument> getDocs() {
		return docs;
	}



	public void setDocs(List<AdressaDocument> docs) {
		this.docs = docs;
	}

	
	public String getNextPage(){
		return nextPageUrl;
	}


	public static void main(String[] args) {
		AdressaSAXParser parser = new AdressaSAXParser();
		parser.parse("http://slt.idi.ntnu.no:15100/cgi-bin/xsearch?query=docdatetime%3arange%28min%2c+2011-02-02%29&hits=10&view=adressa&rpf%5fnavigation%3aenabled=0&offset=10");
		
		List<AdressaDocument> docs = parser.getDocs();
		for(AdressaDocument d: docs){
			System.out.println(d.getTitleString());
			System.out.println(d.getTagString());
		}
	}
}
