package adressa;

import global.GlobalParameters;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.Map.Entry;

import util.FileReaderHelper;
import util.HtmlTagHelper;
import util.IntegerValueComperator;
import util.LogHelper;
import util.ValueComparator;

public class EvaluationCategoriesSelcetor {

	private ArrayList<String> availableCategories = new ArrayList<String>();
	private String path = "resources/adressaClassifier/";
	private HashSet<String> testCategories = new HashSet<String>();

	public EvaluationCategoriesSelcetor(){

	}

	public EvaluationCategoriesSelcetor(File file){
		if(file.isDirectory()){
			File[] categories = file.listFiles();
			for(File category: categories){
				if(category.isDirectory()){
					File[] docs = category.listFiles();
					if(docs.length>4){
						availableCategories.add(category.getName());
					}
				}
			}
		}
	}

	public void removeConflictingCategories(String categoryName){
		availableCategories.remove(categoryName);
		File category = new File(path + categoryName);
		File[] docs = category.listFiles();
		for(File doc:docs){
			String[] contentArr =HtmlTagHelper.adressaContentToArray(doc);
			String tagString = contentArr[0];
			String[] tags = tagString.split(";");
			for(String tag: tags){
				availableCategories.remove(tag);
			}
		}
	}

	public void generateTestCategories(){
		while(availableCategories.size()>0 && testCategories.size()<50){
			int nextCategoryIndex = (int)Math.round(Math.random()*availableCategories.size());
			String nextCategory = availableCategories.get(nextCategoryIndex-1);
			testCategories.add(nextCategory);
			removeConflictingCategories(nextCategory);
		}
	}

	public void printTestCategories(){
		for(String s: testCategories){
			System.out.println(s);
		}
		System.out.println(availableCategories.size());
	}

	public static void main(String[] args) throws FileNotFoundException {
		EvaluationCategoriesSelcetor evalSelector = new EvaluationCategoriesSelcetor();
		//		evalSelector.generateTestCategories();
		File dir = new File(GlobalParameters.LOGFILE+evalSelector.getClass().getName());
		if(!dir.isDirectory()){
			dir.mkdir();
		}
		PrintStream ps = new PrintStream(new File(LogHelper.getLogName()));
		System.setOut(ps);
		//		evalSelector.printTestCategories();
		//		
		//		DocumentPlacer dp = new DocumentPlacer();
		//		dp.placeEvaluationDocuments(evalSelector.getTestCategories());
		evalSelector.generateLargestThreeCategories(new File(GlobalParameters.ADRESSA_CLASSIFIER_PATH));
	}

	public HashSet<String> getTestCategories() {
		return testCategories;
	}

	public void setTestCategories(HashSet<String> testCategories) {
		this.testCategories = testCategories;
	}

	public void generateLargestThreeCategories(File file){
		HashMap<String, Integer> largestCategories = new HashMap<String, Integer>();
		if(file.isDirectory()){
			File[] categories = file.listFiles();
			for(File category: categories){
				if(category.isDirectory()){
					largestCategories.put(category.getName(), category.listFiles().length);
				}
			}
		}
		ArrayList<Entry<String, Integer>> arr =  new ArrayList<Entry<String,Integer>>();
		arr.addAll(largestCategories.entrySet());
		Collections.sort(arr, new IntegerValueComperator());
		for(Entry<String, Integer> e: arr){
			System.out.println(e);
		}
	}

}
