package adressa;

import java.io.File;
import java.io.IOException;

import weka.core.Instances;
import weka.core.converters.ArffSaver;
import weka.core.converters.TextDirectoryLoader;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.StringToWordVector;
import mulan.classifier.lazy.MLkNN;
import mulan.data.InvalidDataFormatException;
import mulan.data.MultiLabelInstances;
import mulan.evaluation.Evaluator;
import mulan.evaluation.MultipleEvaluation;

public class MulanClassifier {
	
	private int numOfClasses =0;

	public Instances loadInstances(File file) throws Exception{
		TextDirectoryLoader loader = new TextDirectoryLoader();
		loader.setDirectory(file);
		Instances dataRaw = loader.getDataSet();  // contains the full dataset we wann create train/test sets from
		System.out.println(dataRaw.numClasses());
		numOfClasses=dataRaw.numClasses();
		StringToWordVector filter = new StringToWordVector();
		//		Standardize standardizeFilter = new Standardize();

		String[] filterOptArr = new String[2];
		filterOptArr[0] = "-N";
		filterOptArr[1] = "1";
		filter.setInputFormat(dataRaw);
		filter.setOutputWordCounts(true);
		//		filter.setOptions(filterOptArr);
		String[] optArr = new String[2];
		optArr[0] = "-delimiters";
		optArr[1] = ",";
		filter.getTokenizer().setOptions(optArr);

		Instances dataFiltered;
		dataFiltered = Filter.useFilter(dataRaw, filter);
		return dataFiltered;
	}
	
	public void runClassification(Instances data) throws Exception{
		String xmlFilename = "resources/xmlMulan.xml";
		String arffName = "resources/multiclass.arff";
		ArffSaver saver = new ArffSaver();
		saver.setInstances(data);
		saver.setFile(new File(arffName));
		saver.writeBatch(); 
		
		System.out.println(data.numClasses());

		MultiLabelInstances dataset = new MultiLabelInstances(arffName, xmlFilename);

		Evaluator eval = new Evaluator();
		MultipleEvaluation results;
		MLkNN learner1 = new MLkNN();

		int numFolds = 2;
		results = eval.crossValidate(learner1, dataset, numFolds);
		System.out.println(results);
	}
	
	public void runClassification() throws Exception{
		String xmlFilename = "D:\\Kristian\\Skole\\workspace\\Master\\resources\\xmlMulan.xml";
//		String xmlFilename = "resources/delicious.xml";
		String arffName = "D:\\Kristian\\Skole\\workspace\\Master\\resources\\multiclass.arff";

//		MultiLabelInstances train = new MultiLabelInstances("resources/delicious-train.arff", xmlFilename);
		MultiLabelInstances dataset = new MultiLabelInstances(arffName, xmlFilename);
		

		Evaluator eval = new Evaluator();
		MultipleEvaluation results;
		MLkNN learner1 = new MLkNN();

		int numFolds = 2;
		results = eval.crossValidate(learner1, dataset, numFolds);
		System.out.println(results);
	}
	

	public static void main(String[] args) throws Exception{
		MulanClassifier mc = new MulanClassifier();
//		Instances data = mc.loadInstances(new File("resources/adressa/textRank"));
		mc.runClassification();
	}


}
