package adressa;

import global.GlobalParameters;

import util.LogHelper;
import weka.core.Instances;
import weka.core.SparseInstance;
import weka.core.converters.ConverterUtils.DataSource;
import weka.core.Utils;
import weka.classifiers.Classifier;
import weka.classifiers.Evaluation;

import weka.classifiers.bayes.NaiveBayes;
import weka.classifiers.functions.SMO;
import weka.classifiers.meta.MultiClassClassifier;
import weka.classifiers.trees.J48;
import weka.core.converters.ArffSaver;
import weka.core.converters.TextDirectoryLoader;
import weka.filters.Filter;
import weka.filters.MultiFilter;
import weka.filters.unsupervised.attribute.Standardize;
import weka.filters.unsupervised.attribute.StringToWordVector;
import weka.filters.unsupervised.instance.NonSparseToSparse;

import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Random;

public class MulticlassClassifierAdressa {

	public MulticlassClassifierAdressa(){
	}



	public void runClassifier() throws Exception{

		String method = "wRAKE";

		TextDirectoryLoader loader = new TextDirectoryLoader();
		loader.setDirectory(new File("resources/adressa/" + method + "Train"));
		Instances dataRaw = loader.getDataSet();  // contains the full dataset we wann create train/test sets from
		int seed = 6;          // the seed for randomizing the data
		int folds = 10;         // the number of folds to generate, >=2

		StringToWordVector filter = new StringToWordVector();

		String[] filterOptArr = new String[2];
		filterOptArr[0] = "-N";
		filterOptArr[1] = "1";
		filter.setInputFormat(dataRaw);
		filter.setOutputWordCounts(true);
		filter.setOptions(filterOptArr);
		String[] optArr = new String[2];
		optArr[0] = "-delimiters";
		optArr[1] = ",";
		filter.getTokenizer().setOptions(optArr);

		Instances dataFiltered;
		Instances testDataFiltered;
		dataRaw = convertToSparseInstance(dataRaw);
		loader.setDirectory(new File("resources/adressa/" + method + "Test"));
		Instances testDataRaw = loader.getDataSet();
		testDataRaw = convertToSparseInstance(testDataRaw);
		dataFiltered = Filter.useFilter(dataRaw, filter);
		testDataFiltered = Filter.useFilter(testDataRaw, filter);

		SMO smoClassifier = new SMO();
		smoClassifier.buildClassifier(dataFiltered);

		System.out.println("mcc");

		dataFiltered.delete();

		// Test the model
		Evaluation smoTest = new Evaluation(testDataFiltered);
		smoTest.evaluateModel(smoClassifier, testDataFiltered);

		// Print the result  la Weka explorer:
		String strSummary = smoTest.toSummaryString();
		System.out.println(strSummary);

		// further processing, classification, etc.

	}
	//		}
	//	}

	public Instances convertToSparseInstance(Instances instance) throws Exception{
		Instances source  = instance;
		NonSparseToSparse sp = new NonSparseToSparse();
		sp.setInputFormat(source);
		Instances newData = Filter.useFilter(source, sp);
		return newData;
	}

	public void writeToArff(Instances instances) throws Exception{
		ArffSaver saver = new ArffSaver();
		saver.setInstances(instances);
		saver.setFile(new File("resources/multiclass.arff"));
		saver.writeBatch(); 
	}

	public static void main(String[] args) throws Exception {
		MulticlassClassifierAdressa aClass = new MulticlassClassifierAdressa();
		//		File dir = new File(GlobalParameters.LOGFILE+aClass.getClass().getName());
		//		if(!dir.isDirectory()){
		//			dir.mkdir();
		//		}
		//		PrintStream ps = new PrintStream(new File(LogHelper.getLogName()));
		//		System.setOut(ps);
		aClass.runClassifier();
	}

}




