package classifer.impl;

import global.GlobalParameters;

import java.io.File;
import java.util.HashSet;

import util.FileReaderHelper;

public class PhraseLengthCounter {

	public PhraseLengthCounter(){

	}

	public PhraseLengthCounter(File dir){
		PhraseLengthCounter pc = new PhraseLengthCounter();
		String text = FileReaderHelper.readFileToString(dir);
		System.out.println("Average Phrase Length: " + pc.calculateAveragePhraseLength(dir));
		System.out.println("Average Phrase Length without weight: " + pc.calculateAveragePhraseLengthWithoutWeight(dir));
		System.out.println("Average number of phrases: " + pc.getAveragePhraseNumberWithoutWeight(dir));
	}

	public double calculateAveragePhraseLengthWithoutWeight(File corpus){
		double numberOfDocs = 0;
		double avgLength = 0;
		File[] categories = corpus.listFiles();
		for(File category: categories){
			if(category.isDirectory()){
				File[] documents = category.listFiles();
				for(File doc: documents){
					avgLength += getAveragePhraseLengthWithoutWeight(doc);
					numberOfDocs++;
				}
			}
		}
		return avgLength/numberOfDocs;
	}

	public double calculateAveragePhraseLength(File corpus){
		double numberOfDocs = 0;
		double avgLength = 0;
		File[] categories = corpus.listFiles();
		for(File category: categories){
			if(category.isDirectory()){
				File[] documents = category.listFiles();
				for(File doc: documents){
					avgLength += getAveragePhraseLength(doc);
					numberOfDocs++;
				}
			}
		}
		return avgLength/numberOfDocs;
	}

	public double getAveragePhraseLength(File document){
		double phraseCount = 0;
		double lengthCount = 0;

		String text = FileReaderHelper.readFileToString(document);
		String[] textArr = text.split(",");
		phraseCount = textArr.length;
		for(String s: textArr){
			lengthCount += s.split(" ").length;
		}
		return lengthCount/phraseCount;
	}

	public double getAveragePhraseLengthWithoutWeight(File document){
		double phraseCount = 0;
		double lengthCount = 0;
		HashSet<String> phrases = new HashSet<String>();

		String text = FileReaderHelper.readFileToString(document);
		String[] textArr = text.split(",");
		for(String s: textArr){
			if(!phrases.contains(s)){
				lengthCount += s.split(" ").length;
				phraseCount++;
				phrases.add(s);
			}

		}
		return lengthCount/phraseCount;
	}

	public double getAveragePhraseNumberWithoutWeight(File corpus){
		HashSet<String> phrases = new HashSet<String>();

		File[] categories = corpus.listFiles();
		for(File category: categories){
			if(category.isDirectory()){
				File[] documents = category.listFiles();
				for(File doc: documents){

					String text = FileReaderHelper.readFileToString(doc);
					String[] textArr = text.split(",");
					for(String s: textArr){			
						phrases.add(s);						
					}
				}
			}
		}
		return phrases.size();
	}

	public static void main(String[] args) {
		File dir = new File("resources/adressa/textRank");
		PhraseLengthCounter pc = new PhraseLengthCounter();
		System.out.println("Average Phrase Length: " + pc.calculateAveragePhraseLength(dir));
		System.out.println("Average Phrase Length without weight: " + pc.calculateAveragePhraseLengthWithoutWeight(dir));
		System.out.println("Average number of phrases: " + pc.getAveragePhraseNumberWithoutWeight(dir));

	}

}
