package classifer.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import keyphraseExtraction.Phrase;
import keyphraseExtraction.impl.PhraseImpl;
import keyphraseExtraction.impl.TaggerExtractor;

import taggerAndWordbank.impl.WordbankFromSerializedParser;

public class Stemmer {
	
	private HashMap<String, String> nounsMap;
	private HashMap<String, String> verbsMap;
	private HashMap<String, String> adjectiveMap;
	private WordbankFromSerializedParser parser;
	private TaggerExtractor tExt;
	
	public Stemmer(TaggerExtractor tExt){
		this.tExt = tExt;
		parser = new WordbankFromSerializedParser();
		try{
			nounsMap = parser.extractNounMap(null);
			verbsMap = parser.extractVerbMap(null);
			adjectiveMap = parser.extractAdjectiveMap(null);
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	
	public Stemmer(){
		parser = new WordbankFromSerializedParser();
		try{
			nounsMap = parser.extractNounMap(null);
			verbsMap = parser.extractVerbMap(null);
			adjectiveMap = parser.extractAdjectiveMap(null);
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	
	public void setTaggerExtractor(TaggerExtractor ext){
		tExt=ext;
	}
	
	public String getBaseForm(String word, String wordClass){
		if(wordClass=="subst"){
			return nounsMap.get(word);
		}
		else if(wordClass=="verb"){
			return verbsMap.get(word);
		}
		else if(wordClass=="adj"){
			return adjectiveMap.get(word);
		}
		return word;
		
	}
	
	public String stemToken(String word, String tag){
		return getBaseForm(word, tag);
	}
	
	public ArrayList<String> stemString(ArrayList<String> sentence, String[] tags){		
		if(sentence.size()!=tags.length){
//			System.err.println("STEMMER.NOT EQUAL LENGTH");
		}
		ArrayList<String> stemmedArray = new ArrayList<String>();
		for(int i=0;i<sentence.size();i++){
			stemmedArray.add(getBaseForm(sentence.get(i), tags[i]));
		}
		return stemmedArray;
	}
	
	public List<Phrase> getStemmedPhrases(List<Phrase> phrases){
		List<Phrase> stemmedPhrases = new ArrayList<Phrase>();
		for(Phrase p: phrases){
			String[] tagArr = tExt.getTags(p);
			ArrayList<String> pText = p.getPhrase();
			if(tagArr.length!=pText.size()){
//				System.err.println("STEMMER.TAGARR AND WORDARR NOT EQUAL LENGTH");
			}
			Phrase newPhrase = new PhraseImpl(stemString(pText, tagArr),p.getScore(),p.getStart(),p.getEnd());
			stemmedPhrases.add(newPhrase);
		}
		return stemmedPhrases;
	}

}
