

package classifer.impl;



import util.CopyFile;
import util.LogHelper;
import weka.core.*;
import weka.core.converters.*;
import weka.core.tokenizers.Tokenizer;
import weka.classifiers.Classifier;
import weka.classifiers.Evaluation;
import weka.classifiers.bayes.NaiveBayes;
import weka.classifiers.functions.SMO;
import weka.classifiers.trees.*;
import weka.filters.*;
import weka.filters.unsupervised.attribute.*;

import global.GlobalParameters;

import java.io.*;
import java.util.StringTokenizer;

/**
 * Example class that converts HTML files stored in a directory structure into 
 * and ARFF file using the TextDirectoryLoader converter. It then applies the
 * StringToWordVector to the data and feeds a J48 classifier with it.
 *
 * @author FracPete (fracpete at waikato dot ac dot nz)
 */
public class WBTextCategorization {

	public WBTextCategorization(){

	}

	public double runTextCategorization(){
		double maxPrecision = 0;
		// convert the directory into a dataset
		try{
			TextCategorization tc = new TextCategorization();

			tc.CopyDocumentsToClassifierFolder();
			tc.CopyDocumentsToTestClassifierFolder();

			TextDirectoryLoader loader = new TextDirectoryLoader();
			//	    loader.setDirectory(new File(args[0]));
			loader.setDirectory(new File("resources/documents/wbClassifier"));
			Instances dataRaw = loader.getDataSet();
			//System.out.println("\n\nImported data:\n\n" + dataRaw);

			// apply the StringToWordVector
			// (see the source code of setOptions(String[]) method of the filter
			// if you want to know which command-line option corresponds to which
			// bean property)

			StringToWordVector filter = new StringToWordVector();
			
			String[] filterOptArr = new String[2];
			filterOptArr[0] = "-N";
			filterOptArr[1] = "1";
			filter.setInputFormat(dataRaw);
			filter.setOutputWordCounts(true);
			filter.setOptions(filterOptArr);
			String[] optArr = new String[2];
			optArr[0] = "-delimiters";
			optArr[1] = ",";
			filter.getTokenizer().setOptions(optArr);
			
//			String[] optArr = new String[2];
//			optArr[0] = "-delimiters";
//			optArr[1] = ",";
//			filter.getTokenizer().setOptions(optArr);
//
//			String[] filterOptArr = new String[2];
//			filterOptArr[0] = "-N";
//			filterOptArr[1] = "1";
//			filter.setInputFormat(dataRaw);
//			filter.setOutputWordCounts(true);
//			filter.setOptions(filterOptArr);
			Instances dataFiltered = Filter.useFilter(dataRaw, filter);

			//train a Naive Bayes and output model
			Classifier nb = (Classifier)new NaiveBayes();
			nb.buildClassifier(dataFiltered);

			// train J48 and output model
			J48 classifier = new J48();
			classifier.buildClassifier(dataFiltered);

			//SOM classifier
			SMO smoClassifier = new SMO();
			smoClassifier.buildClassifier(dataFiltered);

			//TEST THE MODEL PART
			loader.setDirectory(new File("resources/documents/wbTestClassifier"));
			Instances testDataRaw = loader.getDataSet();
			Instances testDataFiltered = Filter.useFilter(testDataRaw, filter);

			// Test the model
			Evaluation nbTest = new Evaluation(testDataFiltered);
			nbTest.evaluateModel(nb, testDataFiltered);
			if(nbTest.correct()>maxPrecision){
				maxPrecision=nbTest.correct();
			}

			// Test the model
			Evaluation j48Test = new Evaluation(testDataFiltered);
			j48Test.evaluateModel(classifier, testDataFiltered);
			if(j48Test.correct()>maxPrecision){
				maxPrecision=j48Test.correct();
			}

			// Test the model
			Evaluation smoTest = new Evaluation(testDataFiltered);
			smoTest.evaluateModel(smoClassifier, testDataFiltered);
			if(smoTest.correct()>maxPrecision){
				maxPrecision=smoTest.correct();
			}

			//				System.out.println(smoTest.correct());
		}
		catch(Exception e){
			e.printStackTrace();
		}
		return maxPrecision;
	}

	public void CopyDocumentsToClassifierFolder(){
		File dir = new File(GlobalParameters.CLASSIFIER_PATH);
		//		File textRankDir = new File(GlobalParamteres.TEXTRANK_CLASSIFIER_PATH);
		String fileName="";
		if(dir.isDirectory()){
			File[] files = dir.listFiles();

			for(File file : files){
				if(file.isDirectory()){

					File newDir = new File("resources/documents/wbClassifier" + "/" + file.getName());
					if(!newDir.isDirectory()){
						newDir.mkdir();
					}

					File[] topic = file.listFiles();
					for(File topicFile : topic){
						fileName = topicFile.getName().substring(0, topicFile.getName().length()-4);
						System.out.println(fileName+"1");
						CopyFile.copyfile("resources/documents/wbTokenized" + "/" + fileName, "resources/documents/wbClassifier" + "/" + file.getName()+ "/" +fileName+".txt");
					}
				}
			}
		}
	}

	public void CopyDocumentsToTestClassifierFolder(){
		File dir = new File(GlobalParameters.TESTDOC_CLASSIFIER_PATH);
		//		File textRankDir = new File(GlobalParamteres.TESTDOC_TEXTRANKCLASSIFIER_PATH);
		String fileName="";
		if(dir.isDirectory()){
			File[] files = dir.listFiles();

			for(File file : files){
				if(file.isDirectory()){

					File newDir = new File("resources/documents/wbTestClassifier" + "/" + file.getName());
					if(!newDir.isDirectory()){
						newDir.mkdir();
					}

					File[] topic = file.listFiles();
					for(File topicFile : topic){
						fileName = topicFile.getName().substring(0, topicFile.getName().length()-4);
						CopyFile.copyfile("resources/documents/wbTokenized" + "/" + fileName, "resources/documents/wbTestClassifier" + "/" + file.getName()+ "/" +fileName+".txt");
					}
				}
			}
		}
	}

	/**
	 * Expects the first parameter to point to the directory with the text files.
	 * In that directory, each sub-directory represents a class and the text
	 * files in these sub-directories will be labeled as such.
	 *
	 * @param args        the commandline arguments
	 * @throws Exception  if something goes wrong
	 */
	public static void main(String[] args) throws Exception {
		// convert the directory into a dataset
		try{
			WBTextCategorization tc = new WBTextCategorization();
			File dir = new File(GlobalParameters.LOGFILE+tc.getClass().getName());
			if(!dir.isDirectory()){
				dir.mkdir();
			}
			PrintStream ps = new PrintStream(new File(LogHelper.getLogName()));
			System.setOut(ps);

			tc.CopyDocumentsToClassifierFolder();
			tc.CopyDocumentsToTestClassifierFolder();

			TextDirectoryLoader loader = new TextDirectoryLoader();
			//	    loader.setDirectory(new File(args[0]));
			loader.setDirectory(new File("resources/documents/wbClassifier"));
			Instances dataRaw = loader.getDataSet();
			//System.out.println("\n\nImported data:\n\n" + dataRaw);

			// apply the StringToWordVector
			// (see the source code of setOptions(String[]) method of the filter
			// if you want to know which command-line option corresponds to which
			// bean property)

			StringToWordVector filter = new StringToWordVector();

			filter.setInputFormat(dataRaw);
			filter.setOutputWordCounts(true);
			String[] filterOptArr = new String[2];
			filterOptArr[0] = "-N";
			filterOptArr[1] = "1";
			filter.setInputFormat(dataRaw);
			filter.setOutputWordCounts(true);
			filter.setOptions(filterOptArr);
			String[] optArr = new String[2];
			optArr[0] = "-delimiters";
			optArr[1] = ",";
			filter.getTokenizer().setOptions(optArr);

			Instances dataFiltered = Filter.useFilter(dataRaw, filter);
			//		System.out.println(dataFiltered.firstInstance().toString());
			//		dataFiltered.

			//System.out.println("\n\nFiltered data:\n\n" + dataFiltered);

			//train a Naive Bayes and output model
			Classifier nb = (Classifier)new NaiveBayes();
			nb.buildClassifier(dataFiltered);
			System.out.println("\n\nClassifier model:\n\n" + nb);

			System.out.println("________________________________________");

			// train J48 and output model
			J48 classifier = new J48();
			classifier.buildClassifier(dataFiltered);
			System.out.println("\n\nClassifier model:\n\n" + classifier);

			System.out.println("________________________________________");

			//SOM classifier
			SMO smoClassifier = new SMO();
			smoClassifier.buildClassifier(dataFiltered);
			System.out.println("\n\nClassifier model:\n\n" + smoClassifier);

			//TEST THE MODEL PART
			loader.setDirectory(new File(GlobalParameters.TESTDOC_CLASSIFIER_PATH));
			Instances testDataRaw = loader.getDataSet();
			Instances testDataFiltered = Filter.useFilter(testDataRaw, filter);

			// Test the model
			Evaluation nbTest = new Evaluation(testDataFiltered);
			nbTest.evaluateModel(nb, testDataFiltered);

			// Print the result  la Weka explorer:
			String strSummary = nbTest.toSummaryString();
			System.out.println(strSummary);

			// Get the confusion matrix
			double[][] cmMatrix = nbTest.confusionMatrix();

			// Test the model
			Evaluation j48Test = new Evaluation(testDataFiltered);
			j48Test.evaluateModel(classifier, testDataFiltered);

			// Print the result  la Weka explorer:
			String strj48TestSummary = j48Test.toSummaryString();
			System.out.println(strj48TestSummary);

			// Test the model
			Evaluation smoTest = new Evaluation(testDataFiltered);
			smoTest.evaluateModel(smoClassifier, testDataFiltered);

			// Print the result  la Weka explorer:
			String strSmoSummary = smoTest.toSummaryString();
			System.out.println(strSmoSummary);
			System.out.println(smoTest.correct());
		}
		catch(Exception e){
			e.printStackTrace();
		}

	}
}






