package keyphraseExtraction.impl;

import java.io.File;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import classifer.impl.PhraseTokenizer;
import clustering.impl.PhraseClusteringWb;

import keyphraseExtraction.Phrase;

import global.GlobalParameters;

import util.HtmlTagHelper;
import util.StringToFile;
import weka.core.tokenizers.Tokenizer;

public class Main {

	@SuppressWarnings("unchecked")
	public static void main(String[] args){
		StopwordRemoverImpl remover;
		StopwordReader stopwordReader = new StopwordReader(new File(GlobalParameters.STOP_WORD_LIST));
		NounWBKeywordExtractor safeEx = new NounWBKeywordExtractor();
		PhraseTokenizer phrTokenzier = new PhraseTokenizer();
		
		File dir = new File(GlobalParameters.DOCUMENT_PATH);
		if(dir.isDirectory()){
			File[] files = dir.listFiles();

			for(File file : files){
				if(file.isDirectory()){
					continue;
				}
				try{
					remover = new StopwordRemoverImpl(stopwordReader.getStopwordMap());
					remover.cleanUp();
					String[] contentArray = HtmlTagHelper.contentToArray(file);
					ArrayList<String[]> contentArrayList = new ArrayList<String[]>();
					for(String part: contentArray){
						String[] text =part.split(" ");
						contentArrayList.add(text);
					}
					//					ArrayList<String> fileStringArray = FileToString.readStringArrayFromFile(file.getAbsolutePath());
					//					remover.removeStopWords(fileStringArray.get(0), 1);
					//					remover.setWordDegrees(1.0);

					//Title
					remover.removeStopWords(contentArray[0]);
					remover.setWordDegrees(3.0);

					//Header
					remover.removeStopWords(contentArray[1]);
					remover.setWordDegrees(5.0);

					//body
					remover.removeStopWords(contentArray[2]);
					remover.setWordDegrees(1.0);

					remover.setRatio();
					remover.calculateRatioKeywords();					

					HashSet<Phrase> keywords = remover.getKeywords();
//					List<String> entries = new ArrayList(keywords.entrySet());
//					Collections.sort(entries, new EntryValueComparator());
//
//					System.out.println(file.getName());

					String s = "";
					Phrase temp;

					List<Phrase> phrases = new ArrayList<Phrase>();
					Iterator i = keywords.iterator();
					
//					List<Candidate> candidates = remover.getCandidates();
//					HashMap<String,Candidate> candidateMap = new HashMap<String, Candidate>();
//					for(Candidate c: candidates){
//						candidateMap.put(c.getCandidateString(), c);
//					}
					
					while ( i.hasNext() )
					{
						temp = (Phrase)i.next();
						String key = (String)temp.getPhraseToString();
//						System.out.println(key);
						
//						Candidate c = candidateMap.get(key);
//						if(ntExtractor.containsNoun(c)){
//							System.out.println("adding: "  + c.getCandidateString());
//							key=key.trim();
//							phrases.add(new PhraseImpl(key, (Double)temp.getValue()));
//						}
						
						if(safeEx.containsNoun(key)){
							key=key.trim();
							phrases.add(new PhraseImpl(temp.getPhrase(), temp.getScore()));
						}
					}

					PhraseClusteringWb pClustering = new PhraseClusteringWb();
//					System.out.println("MAIN.keywords size: " + keywords.size());
					phrases = pClustering.cluster(phrases, safeEx);
					List<Phrase> phrasesWithEndNoun = new ArrayList<Phrase>();
					for(Phrase p: phrases){
						if(safeEx.lastWordIsNoun(p.getPhraseToString())){
							phrasesWithEndNoun.add(p);
						}
					}
					
					
					s = pClustering.toString(phrasesWithEndNoun);
					
					String tokenString = phrTokenzier.makeTokens(phrases);
					StringToFile.writeToFile(tokenString, "wbTokenized\\" + file.getName());
				}
				catch(Exception e){
					e.printStackTrace();
				}
			}
		}
	}
}



