package keyphraseExtraction.impl;


import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;

import clustering.impl.PhraseClusteringWb;

import keyphraseExtraction.Candidate;
import keyphraseExtraction.Phrase;

import global.GlobalParameters;

import util.HtmlTagHelper;
import util.PhraseComparator;

public class ReportMain {

	@SuppressWarnings("unchecked")
	public static void main(String[] args){
		StopwordRemoverImpl remover;
		StopwordReader stopwordReader = new StopwordReader(new File(GlobalParameters.STOP_WORD_LIST));
		NounWBKeywordExtractor safeEx = new NounWBKeywordExtractor();

		File file = new File("resources/documents/misc/report.txt");
		if(file==null){
			System.out.println("FILE NOT FOUND");
		}
		else{
			System.out.println("FILE FOUND");
		}

		//		File[] files = dir.listFiles();

		//		for(File file : files){
		//			if(file.isDirectory()){
		//				continue;
		//			}
		try{
			remover = new StopwordRemoverImpl(stopwordReader.getStopwordMap());
			remover.cleanUp();
			String[] contentArray = HtmlTagHelper.contentToArray(file);
			//			String content = FileReaderHelper.readFileToString(file);

			//Title
			remover.removeStopWords(contentArray[0]);
			remover.setWordDegrees(3.0);

			//Header
			remover.removeStopWords(contentArray[1]);
			remover.setWordDegrees(5.0);

			//body
			remover.removeStopWords(contentArray[2]);
			remover.setWordDegrees(1.0);

			//			remover.removeStopWords(content);
			//			remover.setWordDegrees(1.0);

			System.out.println("OCCURENCEMAP");

			HashMap<String, Double> occurenceMap = remover.getOccurrenceMap();
			for(Entry e: occurenceMap.entrySet()){
				System.out.print(e + ", ");
			}

			System.out.println("DEGREEMAP");

			HashMap<String, Double> degreeMap = remover.getDegreeMap();
			for(Entry e: degreeMap.entrySet()){
				System.out.print(e + ", ");
			}

			System.out.println("CANDIDATES");

			List<Candidate> candidates= remover.getCandidates();
			for(Candidate c: candidates){
				System.out.print(c.getCandidateStringToString()+ " - " );
			}

			remover.setRatio();
			remover.calculateRatioKeywords();					

			HashSet<Phrase> keywords = remover.getKeywords();

			String s = "";
			Phrase temp;

			List<Phrase> phrases = new ArrayList<Phrase>();

			List<Phrase> keywordList = new ArrayList<Phrase>();
			keywordList.addAll(keywords);
			Collections.sort(keywordList, new PhraseComparator());
			Iterator i = keywordList.iterator();



			System.out.println("KEYPHRASES WITH SCORE");
			int counter = 0;

			while ( i.hasNext() )
			{
				temp = (Phrase)i.next();
				String key = (String)temp.getPhraseToString();


				if(counter<10){
					System.out.print(temp.getPhraseToString() + " " + temp.getScore() + ", ");
				}
				counter++;

				if(safeEx.containsNoun(key)){
					key=key.trim();
					phrases.add(new PhraseImpl(temp.getPhrase(), temp.getScore()));
				}
			}

			PhraseClusteringWb pClustering = new PhraseClusteringWb();
			phrases = pClustering.cluster(phrases, safeEx);
			List<Phrase> phrasesWithEndNoun = new ArrayList<Phrase>();
			for(Phrase p: phrases){
				if(safeEx.lastWordIsNoun(p.getPhraseToString())){
					phrasesWithEndNoun.add(p);
				}
			}

			s = pClustering.toString(phrasesWithEndNoun);
			//					StringToFile.writeToFile(s, "keywordsWithProperNounParser\\" + file.getName());
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
}








