package keyphraseExtraction.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

import keyphraseExtraction.Candidate;
import keyphraseExtraction.Phrase;

import util.StringHelper;
import global.GlobalParameters;

public class StopwordRemoverImpl {

	public static final String STOP_WORD_LIST_PATH = "resources\\stopwordsNO";
	private static final String naiveSplit = "\\s+";

	//Hashmap for stoppord
	private HashSet<String> stopwordSet;
	private HashMap<String, Double> occurenceMap;
	private HashMap<String, Double> degreeMap;
	private HashMap<String, Double> ratioMap;
	private List<Candidate> candidateKeywords;
	private HashSet<Phrase> keywords;
	private HashSet<String> stopwordTrigrams;
	
	private int offsetIndex = 0;
	
	private ArrayList<String> candidate;

	public StopwordRemoverImpl(HashSet<String> stopwordSet){
		this.stopwordSet = stopwordSet;
		occurenceMap = new HashMap<String, Double>();
		degreeMap = new HashMap<String, Double>();
		ratioMap = new HashMap<String, Double>();
		keywords = new HashSet<Phrase>();
		stopwordTrigrams = new HashSet<String>();
		
		offsetIndex = 0;

	}
	
	public void cleanUp(){
		candidateKeywords = new ArrayList<Candidate>();
	}

	public void removeStopWords(String text) {		
		
		String[] words = prepareString(text);

		int index = 0;
		int max = words.length;
		
		String temp;
		candidate  = new ArrayList<String>();
		int candidateStart = 0;

		while (index<max){
			temp = words[index];
			if(temp.isEmpty()){			
				index++;
				continue;
			}
			if(candidate.isEmpty()){
				candidateStart = index;
			}
			if(StringHelper.isNoLetters(temp)){
				addCandidate(candidateStart, index);			
				index++;
				continue;
			}
			char last = temp.charAt(temp.length()-1);
			char first = temp.charAt(0);
			if(!Character.isLetterOrDigit(first)){
				temp=temp.substring(1);
				addCandidate(candidateStart, index);
			}
			if (stopwordSet.contains(temp)){
				addToOccurenceMap(temp);
				if(index>0){
					String preword = words[index-1];
					if(!StringHelper.isEndOfPartSentenceorSentence(preword) && index+1<max && GlobalParameters.includeStopwordTrigrams){
						String postword = words[index+1];
						postword = StringHelper.cleanString(postword);
						preword = StringHelper.cleanString(preword);
						ArrayList<String> trigram = new ArrayList<String>();
						trigram.add(preword);
						trigram.add(temp);
						trigram.add(postword);
						String trigramString = preword + " " + temp + " " + postword;
						if(stopwordTrigrams.contains(trigramString)){							
							candidateKeywords.add(new CandidateNewImpl(trigram,index-1, index+1));
						}
						else{
							stopwordTrigrams.add(trigramString);
						}
					}				
				}
				addCandidate(candidateStart, index);
			}
			else if(!Character.isLetterOrDigit(last)){
				if(temp.length()>1){
					temp = temp.substring(0, temp.length()-1);
				}
				candidate.add(temp);
				addToOccurenceMap(temp);
				addCandidate(candidateStart, index);
			}							
			else{
				candidate.add(temp);		
				addToOccurenceMap(temp);	
			}
			index++;
		}
		offsetIndex = max;
	}
	
	public void removeStopWords(String[] text) {
		if(text==null || text.length==0){
			return;
		}
		String[] words = text;

		int index = 0;
		int max = words.length;
		
		String temp;
		candidate  = new ArrayList<String>();
		
		int candidateStart = 0;

		while (index<max){
			try{
			temp = words[index];
			if(temp==null || temp.isEmpty()){			
				index++;
				continue;
			}
			if(candidate.isEmpty()){
				candidateStart = index;
			}
			if(StringHelper.isNoLetters(temp)){
				if(temp=="18"){
					System.out.println("STOPWORDREMOVERIMPL."+words[index-1]);
				}
				if(candidateStart!=index){
					addCandidate(candidateStart, index);	
//					System.out.println("STOPWORDREMOVERIMPL."+temp);
//					System.out.println("STOPWORDREMOVERIMPL."+candidateStart+ " " + index);
				}					
				index++;
				continue;
			}
			char last = temp.charAt(temp.length()-1);
			char first = temp.charAt(0);
			if(!Character.isLetterOrDigit(first)){
				temp=temp.substring(1);
				addCandidate(candidateStart, index);
			}
			if (stopwordSet.contains(temp)){
				addToOccurenceMap(temp);
				if(index>0){				
					String preword = words[index-1];					
					if(!StringHelper.isEndOfPartSentenceorSentence(preword) && index+1<max){
						String postword = words[index+1];
						if(!StringHelper.isNoLetters(preword) && !StringHelper.isNoLetters(postword)){
							postword = StringHelper.cleanString(postword);
							preword = StringHelper.cleanString(preword);
							ArrayList<String> trigram = new ArrayList<String>();
							trigram.add(preword);
							trigram.add(temp);
							trigram.add(postword);
							String trigramString = preword + " " + temp + " " + postword;
							if(stopwordTrigrams.contains(trigramString)){							
								candidateKeywords.add(new CandidateNewImpl(trigram,index-1, index+1));
							}
							else{
								stopwordTrigrams.add(trigramString);
							}
						}
						
					}				
				}
				addCandidate(candidateStart, index);
			}
			else if(!Character.isLetterOrDigit(last)){
				if(temp.length()>1){
					temp = temp.substring(0, temp.length()-1);
				}
				candidate.add(temp);
				addToOccurenceMap(temp);
				addCandidate(candidateStart, index);
			}							
			else{
				candidate.add(temp);		
				addToOccurenceMap(temp);	
			}
			}
			catch(NullPointerException e){
				System.out.println(index);
				e.printStackTrace();
			}
			index++;
		}
		offsetIndex += max;
	}

	public void calculateRatioKeywords(){
		double candidateValue;
		for(Candidate c: candidateKeywords){
//			String[] strings =c.getCandidateString().split(" ");
			ArrayList<String> strings = c.getCandidateString();
			candidateValue = 0.0;
			if(strings.size()<GlobalParameters.MIN_KEYWORD_WORDCOUNT || strings.size()>GlobalParameters.MAX_KEYWORD_WORDCOUNT){
				continue;
			}
			for(String t: strings){
				if(t.isEmpty() || !ratioMap.containsKey(t)) continue;
				candidateValue+=ratioMap.get(t);
			}
//			keywords.put(c.getCandidateString(), candidateValue);
			keywords.add(new PhraseImpl(c.getCandidateString(), candidateValue, c.getStart(), c.getEnd()));
		}
	}


	public void setRatio(){
		double freq;
		double degree;
		double ratio;

		if(GlobalParameters.useRatio){
			Set<String> keys = degreeMap.keySet();
			for(String s: keys){
				try{
					if(occurenceMap.containsKey(s)){
						freq = occurenceMap.get(s);
						degree = degreeMap.get(s);
						ratio = degree/freq;
						ratioMap.put(s, ratio);
					}
					
				}
				catch(NullPointerException e){
					System.out.println("STOPWORDREMOVERIMPL.NULLPOINTER: " + s);
					e.printStackTrace();
				}
				
				
			}
		}
		else{
			ratioMap.putAll(degreeMap);
		}
		
	}

	public void setWordDegrees(double value){
		ArrayList<String> keyword;
		double length;
		for(Candidate c: candidateKeywords){
			keyword = c.getCandidateString();
			length = keyword.size();
			for(String word: keyword){
				if(degreeMap.containsKey(word)){
					degreeMap.put(word, degreeMap.get(word)+length*value); //adds 1 to the number of occurences
				}
				else{
					degreeMap.put(word, length*value); //New occurence
				}		
			}
		}
	}

	private String[] prepareString(String d) {
		d = d.replaceAll("\n", " "); // Remove newlines
		d = d.replaceAll("[ ]{2,}"," "); // Remove multiple spaces
		d = d.toLowerCase();
		return d.split(naiveSplit);
	}

	public HashSet<Phrase> getKeywords(){
		return keywords;
	}

//	public HashSet<String> getStopWords(){
//		return stopwordSet;
//	}
	
	public void addToOccurenceMap(String temp){
		if(occurenceMap.containsKey(temp)){
			occurenceMap.put(temp, occurenceMap.get(temp)+1.0); //adds 1 to the number of occurences
		}
		else{
			occurenceMap.put(temp, 1.0); //New occurence
		}	
	}
	
	public void addCandidate(int startIndex, int endIndex){
		if (!candidate.isEmpty()){
			int start = startIndex+offsetIndex;
			int end = endIndex+offsetIndex;
			candidateKeywords.add(new CandidateNewImpl(candidate, start, end));
//			System.out.println("NEW CANDIDATE: " + candidate + " " + start + " " + end);
//			System.out.println("CANDIDATES SIZE: " + candidateKeywords.size());
			candidate = new ArrayList<String>();
		}
	}
	
	public List<Candidate> getCandidates(){
		return candidateKeywords;
	}
	
	public HashMap<String, Double> getOccurrenceMap (){
		return occurenceMap;
	}
	
	public HashMap<String, Double> getDegreeMap (){
		return degreeMap;
	}
	
	

}
