package keyphraseExtraction.impl;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.Map.Entry;

import classifer.impl.PhraseTokenizer;
import classifer.impl.Stemmer;
import clustering.impl.PhraseClustering;

import keyphraseExtraction.Candidate;
import keyphraseExtraction.Phrase;

import global.GlobalParameters;

import util.FileWriterHelper;
import util.HtmlTagHelper;
import util.PhraseComparator;
import util.StringToFile;

public class TaggerMain {

	private StopwordRemoverImpl remover;
	private StopwordReader stopwordReader;
	private Stemmer stemmer;
	private PhraseTokenizer phrTokenizer;

	public TaggerMain() {

		stopwordReader = new StopwordReader(new File(GlobalParameters.STOP_WORD_LIST));
		stemmer = new Stemmer();
		phrTokenizer = new PhraseTokenizer();
		//		NounWBKeywordExtractor safeEx = new NounWBKeywordExtractor();


	}

	public void runExtraction(){
		TaggedTextSplitter TTSplitter = new TaggedTextSplitter();	
		PhraseClustering pClustering = new PhraseClustering();


//		File dir = new File(GlobalParameters.EVALUATION_DOCS_PATH);
		File dir = new File("resources/adressa/singleCategory");
		File weightedRake = new File("resources/adressa/wRAKE");
		for(File category: weightedRake.listFiles()){
			category.delete();
		}
		
		if(dir.isDirectory()){
			File[] files = dir.listFiles();

			for(File file : files){
				if(file.isDirectory()){
					File newDir = new File(weightedRake + "/" + file.getName());
					if(!newDir.isDirectory()){
						newDir.mkdir();
					}
					File[] taggedTexts = file.listFiles();
					for(File textFile: taggedTexts){
						//					continue;
						//				}
						try{
							remover = new StopwordRemoverImpl(stopwordReader.getStopwordMap());
							//					String[] content = HtmlTagHelper.contentToArray(file);
							String[] content = HtmlTagHelper.contentToArray(textFile);
							TaggerExtractor ntExtractor = new TaggerExtractor();
							ArrayList<String[]> contentArrayList = new ArrayList<String[]>();
							for(String part: content){
								if(part==null || part.isEmpty()){
									contentArrayList.add(null);
									continue;
								}
								//						System.out.println("PART: " + part);
								TTSplitter.split(part);
								String[] tags = TTSplitter.getTagArr();
								String[] text = TTSplitter.getTextArr();
								ntExtractor.addTags(tags);
								contentArrayList.add(text);
							}

							remover.cleanUp();
							//					String[] contentArray = HtmlTagHelper.contentToArray(file);
							//					for(String s: contentArray){
							//						System.out.println("LINE" +s);
							//					}
							//					ArrayList<String> fileStringArray = FileToString.readStringArrayFromFile(file.getAbsolutePath());
							//					remover.removeStopWords(fileStringArray.get(0), 1);
							//					remover.setWordDegrees(1.0);

							//Title
							remover.removeStopWords(contentArrayList.get(0));
							if(GlobalParameters.weighted){
								if(GlobalParameters.lowWeight){
									remover.setWordDegrees(1.5);
								}
								else{
									remover.setWordDegrees(3.0);
								}

							}
							else{
								remover.setWordDegrees(1.0);
							}

							//Header
							remover.removeStopWords(contentArrayList.get(1));
							if(GlobalParameters.weighted){
								if(GlobalParameters.lowWeight){
									remover.setWordDegrees(2.5);
								}
								else{
									remover.setWordDegrees(5.0);
								}
							}
							else{
								remover.setWordDegrees(1.0);
							}

							//body
							remover.removeStopWords(contentArrayList.get(2));
							remover.setWordDegrees(1.0);

							remover.setRatio();
							remover.calculateRatioKeywords();					

							HashSet<Phrase> keywords = remover.getKeywords();
							//					System.out.println("MAIN, KEYWORD SIZE: " + keywords.size());
							//					List<String> entries = new ArrayList(keywords.entrySet());
							//					Collections.sort(entries, new EntryValueComparator());

							//					System.out.println(file.getName());

							String s = "";
							Phrase temp;

							List<Phrase> phrases = new ArrayList<Phrase>();				
							Iterator i = keywords.iterator();


							while ( i.hasNext() )
							{
								temp = (Phrase)i.next();
								if(ntExtractor.containsNoun(temp)){

									phrases.add(temp);
								}
								if(!GlobalParameters.onlyNounPhrases){
									phrases.add(temp);
								}
							}

							//STEMMING
							if(GlobalParameters.stemming && !GlobalParameters.preStemming){
								stemmer.setTaggerExtractor(ntExtractor);
								phrases = stemmer.getStemmedPhrases(phrases);
							}


							if(GlobalParameters.onlyEndOfSentenceNoun){
								List<Phrase> phrasesWithEndNoun = new ArrayList<Phrase>();
								for(Phrase p: phrases){
									if(ntExtractor.lastWordIsNoun(p)){
										phrasesWithEndNoun.add(p);
									}
								}
								if(GlobalParameters.doClustering){
									phrases = pClustering.cluster(phrasesWithEndNoun, ntExtractor);
								}
								else{
									phrases = phrasesWithEndNoun;
								}

							}
							else{
								if(GlobalParameters.doClustering){
									phrases = pClustering.cluster(phrases, ntExtractor);
								}			
							}

							phrases = getTopThirdPhrases(phrases);
							s = pClustering.toString(phrases);
							//					phrases = pClustering.cluster(phrases, ntExtractor);
							//					if(GlobalParamteres.onlyEndOfSentenceNoun){
							//						List<Phrase> phrasesWithEndNoun = new ArrayList<Phrase>();
							//						for(Phrase p: phrases){
							//							if(ntExtractor.lastWordIsNoun(p)){
							//								phrasesWithEndNoun.add(p);
							//							}
							//						}
							//
							//						s = pClustering.toString(phrasesWithEndNoun);
							//					}
							//					else{
							//						s = pClustering.toString(phrases);
							//					}


//							StringToFile.writeToFile(s, "keywordsWithProperNounParser\\" + file.getName());

							String tokenString = phrTokenizer.makeTokens(phrases);
//							StringToFile.writeToFile(tokenString, "tokenizedKeywords\\" + file.getName());
							FileWriterHelper.writeFile(weightedRake + "/" + file.getName() + "/" + textFile.getName(), tokenString);

						}
						catch(Exception e){
							e.printStackTrace();
						}
					}
				}
			}
		}
	}
	
	public List<Phrase> getTopThirdPhrases(List<Phrase> phrases){
		List<Phrase> topPhrases = new ArrayList<Phrase>();
		Collections.sort(phrases, new PhraseComparator());
		int topThird = (int)Math.floor(phrases.size()/3);
		for(int i=0;i<topThird;i++){
			topPhrases.add(phrases.get(i));
		}
		return topPhrases;
	}

	@SuppressWarnings("unchecked")
	public static void main(String[] args){
		TaggerMain tm = new TaggerMain();
		tm.runExtraction();
	}

}






