package run;
import global.GlobalParameters;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.security.acl.Acl;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Map.Entry;

import adressa.AdressaClassifier;

import util.LogHelper;
import util.ValueComparator;

import classifer.impl.PhraseLengthCounter;
import classifer.impl.TextCategorization;

import keyphraseExtraction.impl.TaggerMain;


public class RunWithDifferentOptions {
	
	private ArrayList<Boolean> parameters;
	private HashMap<String, Double> resultMap;
	private TaggerMain tm;

	public RunWithDifferentOptions() {
		// TODO Auto-generated constructor stub
		
	}
	
	public void setUp(){
		resultMap = new HashMap<String, Double>();
		parameters = new ArrayList<Boolean>();
		tm = new TaggerMain();

		parameters.add(GlobalParameters.removeProperNouns);
//		parameters.add(GlobalParameters.includeStopwordTrigrams);
		parameters.add(GlobalParameters.weighted);
		parameters.add(GlobalParameters.lowWeight);
		parameters.add(GlobalParameters.useRatio);
//		parameters.add(GlobalParameters.onlyEndOfSentenceNoun);
		parameters.add(GlobalParameters.onlyNounPhrases);
//		parameters.add(GlobalParameters.doClustering);
//		parameters.add(GlobalParameters.removeLowerRanked);
		parameters.add(GlobalParameters.stemming);
//		parameters.add(GlobalParameters.preStemming);


		for(int i=0;i<parameters.size();i++){
			//			parameters.get(i);
			parameters.set(i, false);
		}
	}
	
	public void setParameters(int n, boolean bool){
		if(n==0){
			parameters.set(n, bool);
			GlobalParameters.setOptions(parameters);
			runTextCategorization();
		}
		else{
			parameters.set(n, bool);
			setParameters(n-1, false);
			setParameters(n-1, true);		
//			GlobalParameters.setOptions(parameters);
//			runTextCategorization();
			
		}
	}
	
	public void iterTest(int n, boolean bool){
		if(n==0){
			parameters.set(n, bool);
			for(boolean b: parameters){
				System.out.print(b + " ");
			}
			System.out.print("at n=0");
			System.out.println();
		}
		else{
			parameters.set(n, bool);
			iterTest(n-1, false);
			iterTest(n-1, true);
		}
	}
	
	public String parametersToString(){
		String s = "";
		for(boolean b: parameters){
			s += b + " ";
		}
		return s;
	}
	
	public void printSortedResults(){
		List<String> entries = new ArrayList(resultMap.entrySet());
		Collections.sort(entries, new ValueComparator());
		File dir = new File(GlobalParameters.LOGFILE + this.getClass().getName());
		if(!dir.isDirectory()){
			dir.mkdir();
		}
		PrintStream ps = null;
		try {
			ps = new PrintStream(new File(LogHelper.getLogName()));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.setOut(ps);
		Iterator i = entries.iterator();
		while(i.hasNext()){
			System.out.println(i.next());
		}
//		System.out.println(entries);
	}
	
	public double runTextCategorization(){
		
		tm.runExtraction();
		AdressaClassifier tc = new AdressaClassifier();
		double score = 0.0;
		try {
			score = tc.runClassifier();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
		resultMap.put(parametersToString(), score);
		return score;
	}
	
	public void singleRun(ArrayList<Boolean> options){
		resultMap = new HashMap<String, Double>();
//		parameters = new ArrayList<Boolean>();
		tm = new TaggerMain();
		GlobalParameters.setOptions(options);
		tm.runExtraction();
		AdressaClassifier aClass = new AdressaClassifier();
		File dir = new File(GlobalParameters.LOGFILE+aClass.getClass().getName());
		if(!dir.isDirectory()){
			dir.mkdir();
		}
		PrintStream ps = null;
		try {
			ps = new PrintStream(new File(LogHelper.getLogName()));
		} catch (FileNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		System.setOut(ps);
		GlobalParameters.printOptions();
		try {
			aClass.runClassifier();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
//		System.out.println(runTextCategorization());
	}
	
	public void singleRun(){
		resultMap = new HashMap<String, Double>();
		parameters = new ArrayList<Boolean>();
		tm = new TaggerMain();
		tm.runExtraction();
		AdressaClassifier aClass = new AdressaClassifier();
		
		GlobalParameters.printOptions();
		try {
			aClass.runClassifier();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
//		System.out.println(runTextCategorization());
	}
	
	public ArrayList<Boolean> getGlobalOptions(String optString){
		String[] optArr = optString.split(" ");
		ArrayList<Boolean> options = new ArrayList<Boolean>();
		for(String s: optArr){
			options.add(Boolean.valueOf(s));
		}
		return options;
	}

	public static void main(String[] args) {
		RunWithDifferentOptions run = new RunWithDifferentOptions();
		File dir = new File(GlobalParameters.LOGFILE+run.getClass().getName());
		if(!dir.isDirectory()){
			dir.mkdir();
		}
		PrintStream ps = null;
		try {
			ps = new PrintStream(new File(LogHelper.getLogName()));
		} catch (FileNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		System.setOut(ps);
		System.out.println("WRAKE BEST");
		String optionsString = "true false false false true false";	
		run.singleRun(run.getGlobalOptions(optionsString));
		PhraseLengthCounter pc = new PhraseLengthCounter(new File("resources/adressa/wRAKE"));
		
		System.out.println("RAKE degree");
		optionsString = "false false false false false false";
		run.singleRun(run.getGlobalOptions(optionsString));
		pc = new PhraseLengthCounter(new File("resources/adressa/wRAKE"));
		
		System.out.println("RAKE ratio");
		optionsString = "false false true false false false";
		run.singleRun(run.getGlobalOptions(optionsString));
		pc = new PhraseLengthCounter(new File("resources/adressa/wRAKE"));
		
		System.out.println("WRAKE 2nd");
		optionsString = "true false false false false false";
		run.singleRun(run.getGlobalOptions(optionsString));
		pc = new PhraseLengthCounter(new File("resources/adressa/wRAKE"));
		
		System.out.println("WRAKE 3rd");
		optionsString = "true true true false true false";
		run.singleRun(run.getGlobalOptions(optionsString));
		pc = new PhraseLengthCounter(new File("resources/adressa/wRAKE"));
		
//		run.singleRun();
		
		
		
		
		
		
		
		
		
		
		
		
		
		
//		run.setUp();
//		run.setParameters(run.parameters.size()-1, true);
//		run.setParameters(run.parameters.size()-1, false);
//		run.printSortedResults();
//		ArrayList<Boolean> para = new ArrayList<Boolean>();
//		para.add(false);
//		para.add(false);
//		para.add(false);
//		RunWithDifferentOptions rwdo = new RunWithDifferentOptions();
//		rwdo.parameters = para;
//		System.out.println(para.size());
//		rwdo.iterTest(para.size()-1, true);
//		rwdo.iterTest(para.size()-1, false);
	}


}
