package stopword;

import global.GlobalParameters;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;

import util.LogHelper;
import util.ValueComparator;

public class OddsRatio {

	private HashMap<String, Integer> globalOccurenceMap;
	private List<OddsRatioEntity> oddsCategories;
	private int globalNumberOfDocs;
	private HashMap<String, Double> globalOddsRatioMap;

	public OddsRatio(){
		globalOccurenceMap = new HashMap<String, Integer>();
		oddsCategories = new ArrayList<OddsRatioEntity>();
		globalOddsRatioMap = new HashMap<String, Double>();
	}

	public void calculateOddsRatio(File corpus){
		File[] categories = corpus.listFiles();
		for(File category: categories){
			if (category.isDirectory()){
				continue;
			}
			OddsRatioCategory oddCategory = new OddsRatioCategory(category);
			oddsCategories.add(oddCategory);
			addToGlobalMap(oddCategory.getOccurenceMap());
		}
		globalNumberOfDocs = globalOccurenceMap.size();
		for(OddsRatioEntity category: oddsCategories){
			calculateProbabilities(category);
			addToGlobalOddsRatioMap(category);
		}
		printSortedResults();
	}

	public void calculateOddsRatioDoc(File corpus){
		File[] docs = corpus.listFiles();
		for(File doc: docs){
			if(doc.isDirectory()){
				continue;
			}
			OddsRatioDocument oddDoc = new OddsRatioDocument(doc);
			oddsCategories.add(oddDoc);
			addToGlobalMap(oddDoc.getOccurenceMap());
		}
		globalNumberOfDocs = globalOccurenceMap.size();
		for(OddsRatioEntity doc: oddsCategories){
			calculateProbabilities(doc);
			addToGlobalOddsRatioMap(doc);
		}
		printSortedResults();
	}

	public void addToGlobalOddsRatioMap(OddsRatioEntity category){
		double sizeRatio = (double)category.getNumberOfDocs()/(double)globalNumberOfDocs;
		for(Entry<String, Double> e: category.getOddsProbMap().entrySet()){
			if(globalOddsRatioMap.containsKey(e.getKey())){
				globalOddsRatioMap.put(e.getKey(), globalOddsRatioMap.get(e.getKey())+(e.getValue()*sizeRatio));
//				if(globalOddsRatioMap.get(e.getKey())>5000.0){
//					System.out.println();
//				}
			}
			else{
				globalOddsRatioMap.put(e.getKey(), e.getValue()*sizeRatio);
//				if(globalOddsRatioMap.get(e.getKey())>5000.0){
//					System.out.println();
//				}
			}
		}
	}

	public void calculateProbabilities(OddsRatioEntity category){
		HashMap<String, Integer> occurenceMap = category.getOccurenceMap();
		HashMap<String, Integer> tempMap = getTempMap(occurenceMap);
		HashMap<String, Double> oddsRatioMap = new HashMap<String, Double>();

		HashMap<String, Double> aMap = getPositivProbabilities(occurenceMap, category.getNumberOfDocs());
		HashMap<String, Double> cMap = getNegativeProbabilities(occurenceMap, category.getNumberOfDocs());
		HashMap<String, Double> bMap = getPositivProbabilities(tempMap, globalNumberOfDocs);
		HashMap<String, Double> dMap = getNegativeProbabilities(tempMap, globalNumberOfDocs);

		//		category.setProbAMap(aMap);
		//		category.setProbCMap(cMap);
		//
		//		category.setProbBMap(bMap);
		//		category.setProbDMap(dMap);

		for(String s: occurenceMap.keySet()){
			double a = aMap.get(s);
			double b = bMap.get(s);
			double c = cMap.get(s);
			double d = dMap.get(s);

			double ratio = (a*d)/(b*c);
			oddsRatioMap.put(s, ratio);
		}
		category.setOddsProbMap(oddsRatioMap);
	}

	public HashMap<String, Integer> getTempMap(HashMap<String, Integer> occurenceMap){
		HashMap<String, Integer> tempMap = new HashMap<String, Integer>();
		tempMap.putAll(globalOccurenceMap);
		for(Entry<String, Integer> e: occurenceMap.entrySet()){
			//			try{
			int value = tempMap.get(e.getKey());

			tempMap.put(e.getKey(), value-e.getValue());
			//			}catch(Exception ex){
			//				ex.printStackTrace();
			//				System.out.println("key: " + e.getKey());
			//				System.out.println("value: " + e.getValue());
			//				System.out.println("size of tempMap: " + tempMap.size());
			//			}

		}
		return tempMap;
	}

	/**
	 * Probability that a term is in the category 
	 * @param occurenceMap
	 * @param numberOfDocs
	 * @return
	 */
	public HashMap<String, Double> getPositivProbabilities(HashMap<String, Integer> occurenceMap, int numberOfDocs){
		HashMap<String, Double> aProb = new HashMap<String, Double>();
		for(Entry<String, Integer> e: occurenceMap.entrySet()){
			aProb.put(e.getKey(), (double)e.getValue()/(double)numberOfDocs);
		}
		return aProb;
	}

	/**
	 * Probability that a term is not in a category
	 * @param occurenceMap
	 * @param numberOfDocs
	 * @return
	 */
	public HashMap<String, Double> getNegativeProbabilities(HashMap<String, Integer> occurenceMap, int numberOfDocs){
		HashMap<String, Double> aProb = new HashMap<String, Double>();
		for(Entry<String, Integer> e: occurenceMap.entrySet()){
			aProb.put(e.getKey(), 1-((double)e.getValue()/(double)numberOfDocs));
		}
		return aProb;
	}

	public void addToGlobalMap(HashMap<String, Integer> occurenceMap){
		for(Entry<String, Integer> e: occurenceMap.entrySet()){
			if(globalOccurenceMap.containsKey(e.getKey())){
				globalOccurenceMap.put(e.getKey(), globalOccurenceMap.get(e.getKey())+e.getValue());
			}
			else{
				globalOccurenceMap.put(e.getKey(), e.getValue());
			}
		}
	}

	public void printSortedResults(){
		List<String> entries = new ArrayList(globalOddsRatioMap.entrySet());
		System.out.println(entries.size());
		Collections.sort(entries, new ValueComparator());
		File dir = new File(GlobalParameters.LOGFILE + this.getClass().getName());
		if(!dir.isDirectory()){
			dir.mkdir();
		}
		PrintStream ps = null;
		try {
			ps = new PrintStream(new File(LogHelper.getLogName()));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.setOut(ps);
		Iterator i = entries.iterator();
		while(i.hasNext()){
			System.out.println(i.next());
		}
		//		System.out.println(entries);
	}
	
	public void outputStopwordList(){
		ArrayList<Entry<String, Double>> entries = new ArrayList<Entry<String, Double>>(globalOddsRatioMap.entrySet());
		Collections.sort(entries, new ValueComparator());
		File dir = new File(GlobalParameters.LOGFILE + this.getClass().getName());
		if(!dir.isDirectory()){
			dir.mkdir();
		}
		PrintStream ps = null;
		try {
			ps = new PrintStream(new File(LogHelper.getLogName()));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.setOut(ps);
		int count = 0;
		Iterator<Entry<String, Double>> i = entries.iterator();
		while(i.hasNext() && count<300){
			System.out.println(i.next().getKey());
			count++;
		}
	}

	public static void main(String[] args) {
		OddsRatio or = new OddsRatio();
		or.calculateOddsRatioDoc(new File(GlobalParameters.ADRESSA_DOCUMENT_PATH));
		
	}

}
