package stopword;

import java.io.File;
import java.util.HashMap;
import java.util.HashSet;

import util.FileReaderHelper;
import util.HtmlTagHelper;

public class OddsRatioCategory implements OddsRatioEntity {

	private HashMap<String, Integer> occurenceMap;
	private int numberOfDocs; 
	
	private HashMap<String, Double> probAMap;
	private HashMap<String, Double> probBMap;
	private HashMap<String, Double> probCMap;
	private HashMap<String, Double> probDMap;
	
	private HashMap<String, Double> oddsProbMap;
	
	
	public OddsRatioCategory(File dir){
		occurenceMap = new HashMap<String, Integer>();
		probAMap = new HashMap<String, Double>();
		probBMap = new HashMap<String, Double>();
		probCMap = new HashMap<String, Double>();
		probDMap = new HashMap<String, Double>();
		oddsProbMap = new HashMap<String, Double>();
		calculateRatio(dir);
	}

	public void calculateRatio(File dir){
		if(!dir.isDirectory()){
			return;
		}
		File[] files = dir.listFiles();
		numberOfDocs = files.length;
		
		for(File doc: files){
			HashSet<String> terms = getTerms(doc);
			for(String s: terms){
				addToOccurenceMap(s);
			}
		}
	}
	
	public void addToOccurenceMap(String s){
		if(occurenceMap.containsKey(s)){
			occurenceMap.put(s, occurenceMap.get(s)+1);
		}
		else{
			occurenceMap.put(s, 1);
		}
	}
	
	public HashSet<String> getTerms(File file){
		HashSet<String> termSet = new HashSet<String>();
		String text = FileReaderHelper.readFileToString(file);
		text = HtmlTagHelper.removeTitleAndHeader(text);
		text = text.toLowerCase();
		String[] textArr = text.split(" ");
		for(String s: textArr){
			if(s.contains("/")){
				s= s.substring(0, s.indexOf("/"));
			}
			termSet.add(s);
		}
		return termSet;
	}
	
	public HashMap<String, Integer> getOccurenceMap() {
		return occurenceMap;
	}

	public void setOccurenceMap(HashMap<String, Integer> occurenceMap) {
		this.occurenceMap = occurenceMap;
	}

	public int getNumberOfDocs() {
		return numberOfDocs;
	}

	public void setNumberOfDocs(int numberOfDocs) {
		this.numberOfDocs = numberOfDocs;
	}
	
	public HashMap<String, Double> getProbAMap() {
		return probAMap;
	}

	public void setProbAMap(HashMap<String, Double> probAMap) {
		this.probAMap = probAMap;
	}

	public HashMap<String, Double> getProbBMap() {
		return probBMap;
	}

	public void setProbBMap(HashMap<String, Double> probBMap) {
		this.probBMap = probBMap;
	}

	public HashMap<String, Double> getProbCMap() {
		return probCMap;
	}

	public void setProbCMap(HashMap<String, Double> probCMap) {
		this.probCMap = probCMap;
	}

	public HashMap<String, Double> getProbDMap() {
		return probDMap;
	}

	public void setProbDMap(HashMap<String, Double> probDMap) {
		this.probDMap = probDMap;
	}
	
	public HashMap<String, Double> getOddsProbMap() {
		return oddsProbMap;
	}

	public void setOddsProbMap(HashMap<String, Double> oddsProbMap) {
		this.oddsProbMap = oddsProbMap;
	}

}
