package stopword;

import java.io.File;
import java.util.HashMap;
import java.util.HashSet;

import util.FileReaderHelper;
import util.HtmlTagHelper;
import util.StringHelper;

public class OddsRatioDocument implements OddsRatioEntity {

	private HashMap<String, Integer> occurenceMap;
	private int numberOfDocs; 

	private HashMap<String, Double> oddsProbMap;

	public OddsRatioDocument(File doc){
		occurenceMap = new HashMap<String, Integer>();
		oddsProbMap = new HashMap<String, Double>();
		calculateRatio(doc);
	}

	/* (non-Javadoc)
	 * @see stopword.OddsRatioEntity#calculateRatio(java.io.File)
	 */
	public void calculateRatio(File doc){
		if(doc.isDirectory()){
			return;
		}
		//		File[] files = dir.listFiles();
		numberOfDocs = 1;


		HashSet<String> terms = getTerms(doc);
		for(String s: terms){
			if(!StringHelper.isNoLetters(s)){
				addToOccurenceMap(s);
			}			
		}
		numberOfDocs = occurenceMap.size();
	}


/* (non-Javadoc)
 * @see stopword.OddsRatioEntity#addToOccurenceMap(java.lang.String)
 */
public void addToOccurenceMap(String s){
	if(occurenceMap.containsKey(s)){
//		occurenceMap.put(s, occurenceMap.get(s)+1);
	}
	else{
		occurenceMap.put(s, 1);
	}
}

/* (non-Javadoc)
 * @see stopword.OddsRatioEntity#getTerms(java.io.File)
 */
public HashSet<String> getTerms(File file){
	HashSet<String> termSet = new HashSet<String>();
	String text = FileReaderHelper.readFileToString(file);
	text = HtmlTagHelper.removeTitleAndHeader(text);
	text = text.toLowerCase();
	String[] textArr = text.split(" ");
	for(String s: textArr){
		if(s.contains("/")){
			s= s.substring(0, s.indexOf("/"));
		}
		termSet.add(s);
	}
	return termSet;
}

/* (non-Javadoc)
 * @see stopword.OddsRatioEntity#getOccurenceMap()
 */
public HashMap<String, Integer> getOccurenceMap() {
	return occurenceMap;
}

/* (non-Javadoc)
 * @see stopword.OddsRatioEntity#setOccurenceMap(java.util.HashMap)
 */
public void setOccurenceMap(HashMap<String, Integer> occurenceMap) {
	this.occurenceMap = occurenceMap;
}

/* (non-Javadoc)
 * @see stopword.OddsRatioEntity#getNumberOfDocs()
 */
public int getNumberOfDocs() {
	return numberOfDocs;
}

/* (non-Javadoc)
 * @see stopword.OddsRatioEntity#setNumberOfDocs(int)
 */
public void setNumberOfDocs(int numberOfDocs) {
	this.numberOfDocs = numberOfDocs;
}


/* (non-Javadoc)
 * @see stopword.OddsRatioEntity#getOddsProbMap()
 */
public HashMap<String, Double> getOddsProbMap() {
	return oddsProbMap;
}

/* (non-Javadoc)
 * @see stopword.OddsRatioEntity#setOddsProbMap(java.util.HashMap)
 */
public void setOddsProbMap(HashMap<String, Double> oddsProbMap) {
	this.oddsProbMap = oddsProbMap;
}
}


