package textRank;

import java.util.Arrays;
import java.util.Comparator;
import java.util.TreeMap;

public class Graph extends TreeMap<String, Node>{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	protected Node[] nodeList = null;
	
	/**
	 * Run through N iterations of the TreeRank algorithm, or until
	 * the standard error converges below a given threshold.
	 */

	public void
	runTextRank ()
	{
		final int max_iterations = this.size();
		nodeList = new Node[this.size()];

		// load the node list

		int j = 0;

		for (Node n1 : this.values()) {
			nodeList[j++] = n1;
		}

		// iterate, then sort and mark the top results

		iterateGraph(max_iterations);
	}
	
	protected void iterateGraph(int max_iterations){
		boolean terminate = false;
		final double[] rank_list = new double[nodeList.length];

		// either run through N iterations, or until the standard
		// error converges below a threshold

		for (int k = 0; k < max_iterations; k++) {
			// calculate the next rank for each node

			for (int i = 0; i < nodeList.length; i++) {
				final Node n1 = nodeList[i];
				double rank = 0.0D;

				for (Node n2 : n1.edges) {
					rank += n2.rank / (double) n2.edges.size();
//					System.out.println("EDGES " + n2.edges.size());
				}

				rank *= Constants.dampingFactor;
				rank += 1.0D - Constants.dampingFactor;

				rank_list[i] = rank;
//				System.out.println("RANK " + rank);
			}


			// swap in new rank values

			for (int i = 0; i < nodeList.length; i++) {
				if(Math.abs(rank_list[i]-nodeList[i].rank)<Constants.threshold){
					terminate = true;
				}
				nodeList[i].rank = rank_list[i];
			}
			
			if(terminate){
				break;
			}

		}
	}
	
	public Node[] getNodes(){
		return nodeList;
	}
	
	public void
	sortResults ()
	{
		Arrays.sort(nodeList,
				new Comparator<Node>() {
			public int compare (Node n1, Node n2) {
				if (n1.rank > n2.rank) {
					return -1;
				}
				else if (n1.rank < n2.rank) {
					return 1;
				}
				else {
					return 0;
				}
			}
		}
		);
	}

}
