package textRank;

/*
	Copyright (c) 2009, ShareThis, Inc. All rights reserved.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are
	met:

 * Redistributions of source code must retain the above copyright
	      notice, this list of conditions and the following disclaimer.

 * Redistributions in binary form must reproduce the above
	      copyright notice, this list of conditions and the following
	      disclaimer in the documentation and/or other materials provided
	      with the distribution.

 * Neither the name of the ShareThis, Inc., nor the names of its
	      contributors may be used to endorse or promote products derived
	      from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
	"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
	LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
	A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
	OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
	LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
	OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import java.util.HashSet;

//	import org.apache.commons.math.util.MathUtils;


/**
 * Implements a node in the TextRank graph, denoting some noun or
 * adjective morpheme.
 *
 * @author paco@sharethis.com
 */

public class
Node
implements Comparable<Node>
{

	/**
	 * Public members.
	 */

	public HashSet<Node> edges = new HashSet<Node>();
	public double rank = 0.0D;
	public String key = null;
	public double oldRank = 0.0D;
	public int wordPos;


	/**
	 * Private constructor.
	 */

	public
	Node (final String key, final int wordPos)
	{
		this.rank = 1.0D;
		this.key = key;
		this.oldRank = 1.0D;
		this.wordPos = wordPos;
	}


	/**
	 * Compare method for sort ordering.
	 */

	public int
	compareTo (final Node that)
	{
		if (this.rank > that.rank) {
			return -1;
		}
		else {
			return 1;
		}
//		else {
//			return this.oldRank.compareTo(that.oldRank);
//		}
	}


	/**
	 * Connect two nodes with a bi-directional arc in the graph.
	 */

	public void
	connect (final Node that)
	{
		this.edges.add(that);
		that.edges.add(this);
	}


	/**
	 * Disconnect two nodes removing their bi-directional arc in the
	 * graph.
	 */

	public void
	disconnect (final Node that)
	{
		this.edges.remove(that);
		that.edges.remove(this);
	}





}


