package textRank;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.Map.Entry;

import keyphraseExtraction.impl.TaggedTextSplitter;

import util.FileReaderHelper;
import util.StringHelper;
import util.ValueComparator;

public class TextRank {

	private String[] tags;
	private String[] textArr;
	private int windowSize;
	private Graph graph;
	private ArrayList<Map.Entry<String, Double>> as;

	@SuppressWarnings("unchecked")
	public TextRank(File file){
		TaggedTextSplitter tts = new TaggedTextSplitter();
		String text = FileReaderHelper.readFileToString(file);
		text = removeTitleAndHeader(text);
		graph = new Graph();
		windowSize=2;


		tts.split(text.trim());

		tags = tts.getTagArr();
		textArr = tts.getTextArr();
		//		System.out.println("LENGTH=" + tags.length);


		for(int i=0;i<tags.length;i++){
			//			System.out.println(tags[i]);
			if(tags[i]!=null && isNounOrAdjective(tags[i])){
				addToGraph(textArr[i], i);
			}
		}
		//		for(Entry<String, Node> e:graph.entrySet()){
		//			System.out.println("Graph String: " + e.getKey());
		//		}

		graph.runTextRank();


		TreeMap<String, Node> topThirdNodes = getTopThirdNodes();
		//		TreeMap<String, Double> keywordMap = new TreeMap<String, Double>((Map<? extends String, ? extends Double>) new NodeComperator());
		//		TreeMap keywordMap = new TreeMap(new NodeComperator());
		HashMap<String, Double> keywordMap = new HashMap<String, Double>();
		keywordMap =	getKeywords(topThirdNodes);
		as = new ArrayList<Map.Entry<String, Double>>(keywordMap.entrySet());
		Collections.sort(as, new ValueComparator());
		//		printMap(as);
	}

	public TreeMap<String, Node> getTopThirdNodes(){
		TreeMap<String, Node> map = new TreeMap<String, Node>();
		//		map.putAll(graph.descendingMap());
		graph.sortResults();
		Node[] nodes = graph.getNodes();
		int topThird = (int)Math.floor(graph.size()/3);
		for(int i=0;i<topThird;i++){
			//			Entry<String, Node> temp = graph.pollFirstEntry();
			Node n = nodes[i];
			map.put(n.key, n);
		}
		return map;
	}

	public HashMap<String, Double> getKeywords(TreeMap<String, Node> topThirdNodes){
		//		TreeMap<String, Double> keywordMap = new TreeMap<String, Double>((Map<? extends String, ? extends Double>) new NodeComperator());
		HashMap<String, Double> keywordMap = new HashMap<String, Double>();
		//		HashMap<String, Double> temp = new HashMap<String, Double>();
		ArrayList<String> temp = new ArrayList<String>();
		ArrayList<Double> tempVal = new ArrayList<Double>();
		//		System.out.println(topThirdNodes.size());
		for(int i=0;i<textArr.length;i++){
			//			try{
			//				if(topThirdNodes.containsKey(textArr[i])){
			//					
			//				}
			//			}catch(NullPointerException e){
			//				System.out.println(i + " " + textArr[i]);
			//			}

			if(textArr[i]!=null && topThirdNodes.containsKey(textArr[i]) && isNounOrAdjective(tags[i])){
				//				temp.put(textArr[i], topThirdNodes.get(textArr[i]).rank);
				temp.add(textArr[i]);
				tempVal.add(topThirdNodes.get(textArr[i]).rank);
				//				temp.add(textArr[i], topThirdNodes.get(textArr[i]).rank);
				//				Entry<String, Double> e= new ;
				//				e.setValue(topThirdNodes.get(textArr[i]).rank);
				//				e.
				//					System.out.println(textArr[i]);
			}	
			else{
				if(!temp.isEmpty()){
					//					System.out.println(temp.size());
					String tempString = "";
					double value = 0.0;
					for(int j=0;j<temp.size()-1;j++){
						tempString += temp.get(j) + " ";

					}
					tempString += temp.get(temp.size()-1);
					for(double d: tempVal){
						value += d;
					}
					tempString.trim();
					keywordMap.put(tempString, value);
					temp.clear();
					tempVal.clear();
				}			
			}
		}
		return keywordMap;
	}

	public ArrayList<String> getEdges(int wordPos){
		ArrayList<String> edges = new ArrayList<String>();
		for(int i=wordPos-1;(i>=wordPos-windowSize) && i>=0;i--){
			if(StringHelper.isEndOfSentence(textArr[i])){
				break;
			}
			if(isNounOrAdjective(tags[i])){
				edges.add(textArr[i]);
			}
		}
		//		for(int i=wordPos+1;i<=wordPos+windowSize && i<textArr.length;i++){
		//			if(StringHelper.isEndOfSentence(textArr[i])){
		//				break;
		//			}
		//			if(isNounOrAdjective(tags[i])){
		//				edges.add(textArr[i]);
		//			}
		//		}
		return edges;
	}

	public void addToGraph(String word, int wordPos){
		ArrayList<String> edges = getEdges(wordPos);
		addNode(word, edges, wordPos);
		//		for(String s: edges){
		//			System.out.println("EDGE " + s);
		//		}
	}


	public void addNode(String word, ArrayList<String> edges,int wordPos){
		Node n;
		if(graph.containsKey(word)){
			n = graph.get(word);
		}
		else{
			n = new Node(word, wordPos);
			graph.put(word, n);
		}
		for(String s: edges){
			Node that = graph.get(s);
			that.connect(n);
		}
	}

	public boolean isNounOrAdjective(String word){
		if(isNoun(word) || isAdjective(word)){
			return true;
		}
		return false;
	}

	public boolean isNoun(String word){
		if(word==null){
			return false;
		}
		return word.equalsIgnoreCase("nn");
	}

	public boolean isAdjective(String word){
		if(word==null){
			return false;
		}
		return word.equalsIgnoreCase("jj");
	}

	public String removeTitleAndHeader(String text){
		text = text.replaceAll("\\*TITLE\\*", "");
		text = text.replaceAll("\\*/TITLE\\*", "");
		text = text.replaceAll("\\*HEADER\\*", "");
		text = text.replaceAll("\\*/HEADER\\*", "");
		text = text.replaceAll("\\s+", " ");
		return text;
	}

	public void printMap(ArrayList<Entry<String, Double>> as){
		Iterator<Entry<String,Double>> i = as.iterator();  
		while ( i.hasNext() )  
		{  
			System.out.println( (Map.Entry<String,Double>)i.next() );  
		}  
	}

	public ArrayList<Entry<String, Double>> getKeywordList(){
		return as;
	}

	public static void main(String[] args) {

		//		TextRank tr = new TextRank(new File("resources/documents/tagged/111-elger-drept-i-ar-1.4581655.txt"));
		TextRank tr = new TextRank(new File("resources/adressaClassifier/Administrasjon/Sakskte NTNU etter 37 kontrakter.txt"));
//		TextRank tr = new TextRank(new File("resources/documents/tagged/100-ledige-jobber-i-fjellregionen-1.4556051.txt"));
		tr.printMap(tr.as);
	}

}
