package textRank;

import global.GlobalParameters;

import java.io.File;
import java.io.PrintStream;

import util.CopyFile;
import util.FileReaderHelper;
import util.FileWriterHelper;
import util.LogHelper;
import weka.classifiers.Classifier;
import weka.classifiers.Evaluation;
import weka.classifiers.bayes.NaiveBayes;
import weka.classifiers.functions.SMO;
import weka.classifiers.trees.J48;
import weka.core.Instances;
import weka.core.Range;
import weka.core.converters.TextDirectoryLoader;
import weka.core.converters.ConverterUtils.DataSource;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.StringToWordVector;
import weka.classifiers.evaluation.Prediction;

public class TextRankCategorization {
	
	public void CopyDocumentsToClassifierFolder(){
		File dir = new File(GlobalParameters.CLASSIFIER_PATH);
		File textRankDir = new File(GlobalParameters.TEXTRANK_CLASSIFIER_PATH);
		String fileName="";
		if(dir.isDirectory()){
			File[] files = dir.listFiles();

			for(File file : files){
				if(file.isDirectory()){
					
					File newDir = new File(textRankDir + "/" + file.getName());
					if(!newDir.isDirectory()){
						newDir.mkdir();
					}
					
					File[] topic = file.listFiles();
					for(File topicFile : topic){
						fileName = topicFile.getName().substring(0, topicFile.getName().length()-4);
						CopyFile.copyfile(GlobalParameters.TEXTRANK_TOKENIZED_KEYWORDS_PATH + "/" + fileName, GlobalParameters.TEXTRANK_CLASSIFIER_PATH + "/" + file.getName()+ "/" +fileName);
					}
				}
			}
		}
	}
	
	public void CopyDocumentsToTestClassifierFolder(){
		File dir = new File(GlobalParameters.TESTDOC_CLASSIFIER_PATH);
		File textRankDir = new File(GlobalParameters.TESTDOC_TEXTRANKCLASSIFIER_PATH);
		String fileName="";
		if(dir.isDirectory()){
			File[] files = dir.listFiles();

			for(File file : files){
				if(file.isDirectory()){
					
					File newDir = new File(textRankDir + "/" + file.getName());
					if(!newDir.isDirectory()){
						newDir.mkdir();
					}
					
					File[] topic = file.listFiles();
					for(File topicFile : topic){
						fileName = topicFile.getName().substring(0, topicFile.getName().length()-4);
						CopyFile.copyfile(GlobalParameters.TEXTRANK_TOKENIZED_KEYWORDS_PATH + "/" + fileName, GlobalParameters.TESTDOC_TEXTRANKCLASSIFIER_PATH + "/" + file.getName()+ "/" +fileName);
					}
				}
			}
		}
	}
	
	public static void main(String[] args) throws Exception{
		
		
		TextRankCategorization TRC = new TextRankCategorization();
		File dir = new File(GlobalParameters.LOGFILE+TRC.getClass().getName());
		if(!dir.isDirectory()){
			dir.mkdir();
		}
		PrintStream ps = new PrintStream(new File(LogHelper.getLogName()));
		System.setOut(ps);
//		TRC.CopyDocumentsToClassifierFolder();
//		TRC.CopyDocumentsToTestClassifierFolder();
		
		TextDirectoryLoader loader = new TextDirectoryLoader();
		loader.setDirectory(new File(GlobalParameters.TEXTRANK_CLASSIFIER_PATH));
		Instances dataRaw = loader.getDataSet();


		StringToWordVector filter = new StringToWordVector();
		
		String[] filterOptArr = new String[2];
		filterOptArr[0] = "-N";
		filterOptArr[1] = "1";
		filter.setInputFormat(dataRaw);
		filter.setOutputWordCounts(true);
		filter.setOptions(filterOptArr);
		String[] optArr = new String[2];
		optArr[0] = "-delimiters";
		optArr[1] = ",";
		filter.getTokenizer().setOptions(optArr);
		
//		String[] optArr = new String[2];
//		optArr[0] = "-delimiters";
//		optArr[1] = ",";
//		filter.getTokenizer().setOptions(optArr);
//
//		filter.setInputFormat(dataRaw);
		
		Instances dataFiltered = Filter.useFilter(dataRaw, filter);
		System.out.println(dataFiltered.firstInstance().toString());
		
		
		//train a Naive Bayes and output model
		Classifier nb = (Classifier)new NaiveBayes();
		nb.buildClassifier(dataFiltered);
		System.out.println("\n\nClassifier model:\n\n" + nb);
		
		System.out.println("________________________________________");

		// train J48 and output model
		J48 classifier = new J48();
		classifier.buildClassifier(dataFiltered);
		System.out.println("\n\nClassifier model:\n\n" + classifier);
		
		System.out.println("________________________________________");
		
		//SOM classifier
		SMO smoClassifier = new SMO();
		smoClassifier.buildClassifier(dataFiltered);
		System.out.println("\n\nClassifier model:\n\n" + smoClassifier);
		
		//TEST THE MODEL PART
		loader.setDirectory(new File(GlobalParameters.TESTDOC_TEXTRANKCLASSIFIER_PATH));
		
		Instances testDataRaw = loader.getDataSet();
		Instances testDataFiltered = Filter.useFilter(testDataRaw, filter);
		
		System.out.println("NAIVE BAYES");
		
		 // Test the model
		 Evaluation nbTest = new Evaluation(testDataFiltered);
		 nbTest.evaluateModel(nb, testDataFiltered);
		 
		 // Print the result  la Weka explorer:
		 String strSummary = nbTest.toSummaryString();
		 System.out.println(strSummary);
		 
		 // Get the confusion matrix
//		 double[][] cmMatrix = nbTest.confusionMatrix();
		 System.out.println(nbTest.toMatrixString());
		 
		 System.out.println(nbTest.toClassDetailsString());
		 DataSource ds = new DataSource(testDataFiltered);
		 StringBuffer sb = new StringBuffer();
		 System.out.println(testDataFiltered.firstInstance().toString());
//		 Evaluation.printClassifications(nb, dataFiltered, ds, 1, new Range("1-20"), sb);
		 System.out.println(sb);
		 
		 System.out.println("J48");
		 
		 // Test the model
		 Evaluation j48Test = new Evaluation(testDataFiltered);
		 j48Test.evaluateModel(classifier, testDataFiltered);
		 
		 // Print the result  la Weka explorer:
		 String strj48TestSummary = j48Test.toSummaryString();
		 System.out.println(strj48TestSummary);
		 
		 System.out.println("SMO");
		 
		 // Test the model
		 Evaluation smoTest = new Evaluation(testDataFiltered);
		 smoTest.evaluateModel(smoClassifier, testDataFiltered);
		 
		 // Print the result  la Weka explorer:
		 String strSmoSummary = smoTest.toSummaryString();
		 System.out.println(strSmoSummary);
		 
		 ps.close();
	}

}
