package unused;

import global.GlobalParameters;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.TreeMap;
import java.util.Map.Entry;
import libsvm.*;

import keyphraseExtraction.Phrase;

import util.FileReaderHelper;

public class LibSvmTest {

	private HashMap<String, Integer> attributeMap;
	private HashMap<String, Integer> labelMap;

	public LibSvmTest(){
		attributeMap = new HashMap<String, Integer>();
		labelMap = new HashMap<String, Integer>();
	}

	public void fillAttributeMap(File corpus){
		int index = 1;
		int labelIndex = 1;
		if(corpus.isDirectory()){
			File[] categories = corpus.listFiles();
			for(File category: categories){
				if(category.isDirectory()){
					labelMap.put(category.getName(), labelIndex);
					labelIndex++;
					File[] documents = category.listFiles();
					for(File document: documents){
						HashSet<String> attributes = new HashSet<String>();
						attributes.addAll(attributeMap.keySet());
						String text = FileReaderHelper.readFileToString(document);
						String[] phrases = text.split(",");
						for(String s: phrases){
							if(s!=null && !s.isEmpty() && !attributes.contains(s)){
								attributes.add(s);
								attributeMap.put(s, index);
								index++;
							}
						}
					}
				}
			}
		}
	}

	public void generateVectors(File corpus){
		FileOutputStream fos = null;
		PrintWriter pw = null;
		try {
			fos = new FileOutputStream(new File("resources/svmVectorsTest.txt"));
			OutputStreamWriter osw = new OutputStreamWriter(fos);
			pw = new PrintWriter(osw);	        
		}
		catch (IOException e) {
			System.err.println(e);
		}
		if(corpus.isDirectory()){
			File[] categories = corpus.listFiles();
			for(File category: categories){
				System.out.println(category.getName());
				if(category.isDirectory()){
					File[] documents = category.listFiles();
					for(File document: documents){
						try{

							pw.println(generateDocumentVector(document, labelMap.get(category.getName())));
						}	
						catch(NullPointerException e){
							System.out.println(document.getName());
							e.printStackTrace();
						}
					}


				}
			}
		}
		pw.close();
	}

	public String generateDocumentVector(File file, int label){
		TreeMap<Integer, Integer> vectorMap = new TreeMap<Integer, Integer>();
		String text = FileReaderHelper.readFileToString(file);
		String[] phrases = text.split(",");
		for(String s: phrases){
			int index = attributeMap.get(s);
			if(vectorMap.containsKey(index)){
				vectorMap.put(index, vectorMap.get(index) + 1);
			}
			else{
				vectorMap.put(index, 1);
			}

		}

		String vector = "" + label;
		for(Entry<Integer, Integer> e: vectorMap.entrySet()){
			vector += " " + e.getKey() + ":" + e.getValue();
		}
		return vector;
	}

	public String generateDocumentVector(List<Phrase> phrases, String label){
		TreeMap<Integer, Double> vectorMap = new TreeMap<Integer, Double>();
		for(Phrase p: phrases){
			String s = p.getPhraseToString();
			int index = attributeMap.get(s);
			vectorMap.put(index, p.getScore());
		}

		String vector = label;
		for(Entry<Integer, Double> e: vectorMap.entrySet()){
			vector += " " + e.getKey() + ":" + e.getValue();
		}
		return vector;
	}

	public static void main(String[] args) {
		LibSvmTest svm = new LibSvmTest();
		System.out.println("Filling attributemap");
		svm.fillAttributeMap(new File("resources/adressa/wRAKETest"));
		System.out.println("Generating vectorfile");
		svm.generateVectors(new File("resources/adressa/wRAKETest"));
	}

}
