package wekaPhraseExtractor;

import global.GlobalParameters;

import java.io.File;
import java.io.PrintStream;

import util.CopyFile;
import util.FileReaderHelper;
import util.FileWriterHelper;
import util.LogHelper;
import weka.classifiers.Classifier;
import weka.classifiers.Evaluation;
import weka.classifiers.bayes.NaiveBayes;
import weka.classifiers.functions.SMO;
import weka.classifiers.trees.J48;
import weka.core.Instances;
import weka.core.Range;
import weka.core.converters.TextDirectoryLoader;
import weka.core.converters.ConverterUtils.DataSource;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.StringToWordVector;

public class WekaTextCategorization {
	
	public WekaTextCategorization(){
		
	}
	
	public void removeTitleAndHeader(File file){
		String text = FileReaderHelper.readFileToString(file);
		text = text.replaceAll("\\*TITLE\\*", "");
		text = text.replaceAll("\\*/TITLE\\*", "");
		text = text.replaceAll("\\*HEADER\\*", "");
		text = text.replaceAll("\\*/HEADER\\*", "");
		text = text.replaceAll("\\s+", " ");
		FileWriterHelper.writeFile(file.getPath(), text);
	}
	
	public void CopyDocumentsToClassifierFolder(){
		File dir = new File(GlobalParameters.CLASSIFIER_PATH);
		File wekaDir = new File(GlobalParameters.WEKA_CLASSIFIER_PATH);
		String fileName="";
		if(dir.isDirectory()){
			File[] files = dir.listFiles();

			for(File file : files){
				if(file.isDirectory()){
					
					File newDir = new File(wekaDir + "/" + file.getName());
					if(!newDir.isDirectory()){
						newDir.mkdir();
					}
					
					File[] topic = file.listFiles();
					for(File topicFile : topic){
						fileName = topicFile.getName().substring(0, topicFile.getName().length()-4);
						CopyFile.copyfile(GlobalParameters.DOCUMENT_PATH + "/" + fileName, GlobalParameters.WEKA_CLASSIFIER_PATH + "/" + file.getName()+ "/" +fileName);
						
//						File replacementFile = new File(GlobalParamteres.DOCUMENT_PATH + "/" + fileName);
//						file = new File(GlobalParamteres.WEKA_CLASSIFIER_PATH + "/" + file.getName());
//						replacementFile.renameTo(new File(file, fileName));
						
						File theNewFile = new File(GlobalParameters.WEKA_CLASSIFIER_PATH + "/" + file.getName()+ "/"+ fileName.substring(0, fileName.length()));
						removeTitleAndHeader(theNewFile);
					}
				}
			}
		}
	}
	
	public void CopyDocumentsToTestClassifierFolder(){
		File dir = new File(GlobalParameters.TESTDOC_CLASSIFIER_PATH);
		File wekaDir = new File(GlobalParameters.TESTDOC_WEKACLASSIFIER_PATH);
		String fileName="";
		if(dir.isDirectory()){
			File[] files = dir.listFiles();

			for(File file : files){
				if(file.isDirectory()){
					
					File newDir = new File(wekaDir + "/" + file.getName());
					if(!newDir.isDirectory()){
						newDir.mkdir();
					}
					
					File[] topic = file.listFiles();
					for(File topicFile : topic){
						fileName = topicFile.getName().substring(0, topicFile.getName().length()-4);
						CopyFile.copyfile(GlobalParameters.DOCUMENT_PATH + "/" + fileName, GlobalParameters.TESTDOC_WEKACLASSIFIER_PATH + "/" + file.getName()+ "/" +fileName);
						
//						File replacementFile = new File(GlobalParamteres.DOCUMENT_PATH + "/" + fileName);
//						file = new File(GlobalParamteres.WEKA_CLASSIFIER_PATH + "/" + file.getName());
//						replacementFile.renameTo(new File(file, fileName));
						
						File theNewFile = new File(GlobalParameters.TESTDOC_WEKACLASSIFIER_PATH + "/" + file.getName()+ "/"+ fileName.substring(0, fileName.length()));
						removeTitleAndHeader(theNewFile);
					}
				}
			}
		}
	}
	
	public static void main(String[] args) throws Exception {
		WekaTextCategorization WTC = new WekaTextCategorization();
		File dir = new File(GlobalParameters.LOGFILE+WTC.getClass().getName());
		if(!dir.isDirectory()){
			dir.mkdir();
		}
		PrintStream ps = new PrintStream(new File(LogHelper.getLogName()));
		System.setOut(ps);
		
		WTC.CopyDocumentsToClassifierFolder();
		WTC.CopyDocumentsToTestClassifierFolder();
		
		TextDirectoryLoader loader = new TextDirectoryLoader();
		//	    loader.setDirectory(new File(args[0]));
		loader.setDirectory(new File("resources/documents/wekaClassifier"));
		Instances dataRaw = loader.getDataSet();
		//System.out.println("\n\nImported data:\n\n" + dataRaw);

		// apply the StringToWordVector
		// (see the source code of setOptions(String[]) method of the filter
		// if you want to know which command-line option corresponds to which
		// bean property)

		StringToWordVector filter = new StringToWordVector();

		filter.setInputFormat(dataRaw);
		Instances dataFiltered = Filter.useFilter(dataRaw, filter);
		System.out.println(dataFiltered.firstInstance().toString());
		
		
		//train a Naive Bayes and output model
		Classifier nb = (Classifier)new NaiveBayes();
		nb.buildClassifier(dataFiltered);
		System.out.println("\n\nClassifier model:\n\n" + nb);
		
		System.out.println("________________________________________");

		// train J48 and output model
		J48 classifier = new J48();
		classifier.buildClassifier(dataFiltered);
		System.out.println("\n\nClassifier model:\n\n" + classifier);
		
		System.out.println("________________________________________");
		
		//SOM classifier
		SMO smoClassifier = new SMO();
		smoClassifier.buildClassifier(dataFiltered);
		System.out.println("\n\nClassifier model:\n\n" + smoClassifier);
		
		
		//TEST THE MODEL PART
		
		loader.setDirectory(new File(GlobalParameters.TESTDOC_WEKACLASSIFIER_PATH));
		Instances testDataRaw = loader.getDataSet();
		filter.setInputFormat(testDataRaw);
		Instances testDataFiltered = Filter.useFilter(testDataRaw, filter);
//		Evaluation.evaluateModel(smoClassifier, data);
		
		 // Test the model
		 Evaluation nbTest = new Evaluation(testDataFiltered);
		 nbTest.evaluateModel(nb, testDataFiltered);
		 
		 // Print the result  la Weka explorer:
		 String strSummary = nbTest.toSummaryString();
		 System.out.println(strSummary);
		 
		 // Get the confusion matrix
//		 double[][] cmMatrix = nbTest.confusionMatrix();
		 System.out.println(nbTest.toMatrixString());
		 
		 System.out.println(nbTest.toClassDetailsString());
		 DataSource ds = new DataSource(testDataFiltered);
		 StringBuffer sb = new StringBuffer();
		 System.out.println(testDataFiltered.firstInstance().toString());
//		 Evaluation.printClassifications(nb, dataFiltered, ds, 1, new Range("1-20"), sb);
		 System.out.println(sb);
		 
		 System.out.println("J48");
		 
//		  Test the model
//		 Evaluation j48Test = new Evaluation(testDataFiltered);
//		 j48Test.evaluateModel(classifier, testDataFiltered);
//		 
//		 // Print the result  la Weka explorer:
//		 String strj48TestSummary = j48Test.toSummaryString();
//		 System.out.println(strj48TestSummary);
		 
//		 // Test the model
//		 Evaluation smoTest = new Evaluation(testDataFiltered);
//		 smoTest.evaluateModel(smoClassifier, testDataFiltered);
//		 
//		 // Print the result  la Weka explorer:
//		 String strSmoSummary = smoTest.toSummaryString();
//		 System.out.println(strSmoSummary);
		 
		 ps.close();

	}

}


